@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2007 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2020 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -25,6 +25,10 @@
 # include <config.h>
 #endif
 
+#if defined(__APPLE__)
+# include <sys/cdefs.h> // defines the system macros checked below
+#endif
+
 #ifndef _GNU_SOURCE
 # define _GNU_SOURCE 1 /* so _XOPEN_SOURCE will be defined by features.h */
 #endif
@@ -34,13 +38,15 @@
 #endif
 
 #if !defined(WIN32)
-#if !defined(_XOPEN_SOURCE) || _XOPEN_SOURCE < 500
-# undef _XOPEN_SOURCE
-# define _XOPEN_SOURCE 500 /* to define PTHREAD_MUTEX_ERRORCHECK */
-# warning "Seems you don't have a UNIX98 compatible system."
-# warning "Please run LinuxSampler's selftest to make sure this won't be a problem!"
-# warning "(compile tests with 'make tests', run them with 'src/testcases/linuxsamplertest')"
-#endif
+# if !defined(_XOPEN_SOURCE) || _XOPEN_SOURCE < 500
+#  undef _XOPEN_SOURCE
+#  define _XOPEN_SOURCE 500 /* to define PTHREAD_MUTEX_RECURSIVE */
+#  if (!defined(POSIX_C_SOURCE) || POSIX_C_SOURCE < 199801L) && !__DARWIN_UNIX03
+#   warning "Seems you don't have a UNIX98 compatible system."
+#   warning "Please run LinuxSampler's selftest to make sure this won't be a problem!"
+#   warning "(compile tests with 'make tests', run them with 'src/testcases/linuxsamplertest')"
+#  endif
+# endif
 #endif
 
 #include <iostream>
@@ -49,9 +55,15 @@
 
 #include "Mutex.h"
 
+#if DEBUG_MUTEX
+# include "Thread.h"
+# include <assert.h>
+# include "global_private.h"
+#endif
+
 namespace LinuxSampler {
 
-Mutex::Mutex() {
+Mutex::Mutex(type_t type) {
 #if defined(WIN32)
     hMutex = CreateMutex( NULL, FALSE, NULL);
     if (hMutex == NULL) {
@@ -60,21 +72,31 @@ Mutex::Mutex() {
     }
 #else
     pthread_mutexattr_init(&__posix_mutexattr);
-
-    // the following function call only works on UNIX98 compatible systems
-    #if (_XOPEN_SOURCE > 500) || defined(__APPLE__)
-	// Mac OS X Note: 10.4 (and later) does support PTHREAD_MUTEX_ERRORCHECK, and
-	// actually LinuxSampler does not work if this call is omitted. However,
-	// defining _XOPEN_SOURCE macro seems to cause other problems. As a workaround,
-	// the symbol __APPLE__ is checked here. There should be a better solution
-	// to this problem. (Toshi Nagata, 27 Mar 2007)
-    if (pthread_mutexattr_settype(&__posix_mutexattr, PTHREAD_MUTEX_ERRORCHECK)) {
-        std::cerr << "Mutex Constructor: Fatal error - unable to pthread_mutexattr_settype(PTHREAD_MUTEX_ERRORCHECK)\n" << std::flush;
+    // pthread_mutexattr_settype() only works on UNIX98 compatible systems
+    switch (type) {
+    case RECURSIVE:
+        if (pthread_mutexattr_settype(&__posix_mutexattr, PTHREAD_MUTEX_RECURSIVE)) {
+            std::cerr << "Mutex Constructor: Fatal error - unable to pthread_mutexattr_settype(PTHREAD_MUTEX_RECURSIVE)\n" << std::flush;
+            exit(-1);
+        }
+        break;
+    case NON_RECURSIVE:
+        if (pthread_mutexattr_settype(&__posix_mutexattr, PTHREAD_MUTEX_ERRORCHECK)) {
+            std::cerr << "Mutex Constructor: Fatal error - unable to pthread_mutexattr_settype(PTHREAD_MUTEX_ERRORCHECK)\n" << std::flush;
+            exit(-1);
+        }
+        break;
+    default:
+        std::cerr << "Mutex Constructor: Fatal error - Unknown mutex type requested\n" << std::flush;
         exit(-1);
+        break;
     }
-    #endif
     pthread_mutex_init(&__posix_mutex, &__posix_mutexattr);
 #endif
+#if DEBUG_MUTEX
+    debugSelf = false;
+    count = 0;
+#endif
 }
 
 Mutex::~Mutex() {
@@ -92,20 +114,56 @@ void Mutex::Lock() {
 #else
     pthread_mutex_lock(&__posix_mutex);
 #endif
+#if DEBUG_MUTEX
+    if (debugSelf) {
+        std::string caller = Thread::nameOfCaller();
+        ++count;
+        assert(count > 0);
+        if (type != RECURSIVE)
+            assert(count == 1);
+        if (!owner.empty())
+            assert(owner == caller);
+        owner = caller;
+        backtrace = backtraceAsString();
+    }
+#endif
 }
 
 bool Mutex::Trylock() {
 #if defined(WIN32)
     if( WaitForSingleObject(hMutex, 0) == WAIT_TIMEOUT) return false;
-    return true;
 #else
     if (pthread_mutex_trylock(&__posix_mutex) == EBUSY)
         return false;
-    return true;
 #endif
+#if DEBUG_MUTEX
+    if (debugSelf) {
+        std::string caller = Thread::nameOfCaller();
+        ++count;
+        assert(count > 0);
+        if (type != RECURSIVE)
+            assert(count == 1);
+        if (!owner.empty())
+            assert(owner == caller);
+        owner = caller;
+        backtrace = backtraceAsString();
+    }
+#endif
+    return true;
 }
 
 void Mutex::Unlock() {
+#if DEBUG_MUTEX
+    if (debugSelf) {
+        std::string caller = Thread::nameOfCaller();
+        assert(count > 0);
+        --count;
+        assert(count >= 0);
+        assert(owner == caller);
+        if (!count)
+            owner.clear();
+    }
+#endif
 #if defined(WIN32)
     ReleaseMutex(hMutex);
 #else
