@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2007 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2019 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -44,15 +44,19 @@ enum implementation_t {
 class RTMathBase {
     public:
         /**
-         * Highly accurate time stamp.
+         * High resolution time stamp.
          */
         typedef uint32_t time_stamp_t;
 
+        typedef uint64_t usecs_t;
+
         /**
          * We read the processor's cycle count register as a reference
          * for the real time. These are of course only abstract values
          * with arbitrary time entity, but that's not a problem as long
          * as we calculate relatively.
+         *
+         * @see unsafeMicroSeconds()
          */
         static time_stamp_t CreateTimeStamp();
 
@@ -95,6 +99,109 @@ class RTMathBase {
             return log(FreqRatio) / log(TWELVEHUNDREDTH_ROOT_OF_TWO);
         }
 
+        /**
+         * Calculates the linear ratio value representation (linear scale)
+         * of the @a decibel value provided (exponential scale).
+         *
+         * The context of audio acoustic sound pressure levels is assumed, and
+         * hence the field version of the dB unit is used here (which uses a
+         * linear factor of 20). This function is a bit slow, so it should
+         * not be called too frequently.
+         *
+         * @param decibel - sound pressure level in dB
+         * @returns linear ratio of the supplied dB value
+         * @see LinRatioToDecibel() as inverse function
+         */
+        static float DecibelToLinRatio(float decibel) {
+            return powf(10.f, decibel / 20.f);
+        }
+
+        /**
+         * Calculates the decibel value (exponential scale) of the @a linear
+         * ratio value representation (linear scale) provided.
+         *
+         * The context of audio acoustic sound pressure levels is assumed, and
+         * hence the field version of the dB unit is used here (which uses a
+         * linear factor of 20). This function is a bit slow, so it should
+         * not be called too frequently.
+         *
+         * @param linear - sound pressure level as linear ratio value (linear scale)
+         * @returns dB value representation
+         * @see DecibelToLinRatio() as inverse function
+         */
+        static float LinRatioToDecibel(float linear) {
+            return 20.f * log10f(linear);
+        }
+
+        /**
+         * Calculates the relatively summed average of a set of values.
+         *
+         * @param current - the current avaerage value of all previously summed values
+         * @param sample - new value to be applied as summed average to the existing values
+         * @param n - amount of sample values applied so far
+         * @returns new average value of all summed values (including the new @a sample)
+         */
+        template<typename T_int>
+        inline static float RelativeSummedAvg(float current, float sample, T_int n) {
+            return current + (sample - current) / float(n);
+        }
+
+        /**
+         * Clock source to use for getting the current time.
+         */
+        enum clock_source_t {
+            real_clock,    ///< Use this to measure time that passed in reality (no matter if process got suspended).
+            process_clock, ///< Use this to measure only the CPU execution time of the current process (if the process got suspended, the clock is paused as well).
+            thread_clock,  ///< Use this to measure only the CPU execution time of the current thread (if the process got suspended or another thread is executed, the clock is paused as well).
+        };
+
+        /**
+         * Returns a time stamp of the current time in microseconds (in
+         * probably real-time @b unsafe way). There is no guarantee about
+         * what the returned amount of microseconds relates to (i.e.
+         * microseconds since epoch, microseconds since system uptime, ...).
+         * So you should only use it to calculate time differences between
+         * values taken with this method.
+         *
+         * @b CAUTION: This method may not @b NOT be real-time safe! On some
+         * systems it could be RT safe, but there is no guarantee whatsoever!
+         * So this method should only be used for debugging, benchmarking and
+         * other developing purposes !
+         *
+         * For creating time stamps in real-time context, use
+         * CreateTimeStamp() instead.
+         *
+         * @param source - the actual clock to use for getting the current
+         *                 time, note that the various clock sources may not
+         *                 be implemented on all systems
+         * @returns time stamp in microseconds
+         *
+         * @see CreateTimeStamp()
+         */
+        static usecs_t unsafeMicroSeconds(clock_source_t source);
+
+        /**
+         * 'Equalness' comparison between two 32 bit floating point numbers
+         * (that is of single precision data type).
+         *
+         *  This methods deals with the expected tolerance of the floating point
+         *  representation.
+         *
+         * @return true if the two numbers can be expected to be equal
+         */
+        static bool fEqual32(float a, float b);
+
+        /**
+         * 'Equalness' comparison between two 64 bit floating point numbers
+         * (that is of double precision data type).
+         *
+         *  This methods deals with the expected tolerance of the floating point
+         *  representation.
+         *
+         * @return true if the two numbers can be expected to be equal
+         */
+        static bool fEqual64(double a, double b);
+
     private:
         static float* pCentsToFreqTable;
 
