@@ -1,6 +1,6 @@
 /***************************************************************************
  *                                                                         *
- *   Copyright (C) 2007-2009 Grigor Iliev, Benno Senoner                   *
+ *   Copyright (C) 2007-2013 Grigor Iliev, Benno Senoner                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -41,12 +41,17 @@ namespace LinuxSampler {
     InstrumentsDb InstrumentsDb::instance;
 
     void InstrumentsDb::CreateInstrumentsDb(String FilePath) {
+        if (FilePath.empty()) {
+            FilePath = GetDefaultDBLocation();
+            dmsg(0,("InstrumentsDb: Creating database at default location '%s'\n", FilePath.c_str()));
+        }
+
         File f = File(FilePath);
         if (f.Exist()) {
             throw Exception("File exists: " + FilePath);
         }
-        
-        GetInstrumentsDb()->SetDbFile(FilePath);
+
+        SetDbFile(FilePath);
 
         String sql = 
             "  CREATE TABLE instr_dirs (                                      "
@@ -60,10 +65,10 @@ namespace LinuxSampler {
             "      UNIQUE (parent_dir_id,dir_name)                            "
             "  );                                                             ";
         
-        GetInstrumentsDb()->ExecSql(sql);
+        ExecSql(sql);
 
         sql = "INSERT INTO instr_dirs (dir_id, parent_dir_id, dir_name) VALUES (0, -2, '/');";
-        GetInstrumentsDb()->ExecSql(sql);
+        ExecSql(sql);
 
         sql =
             "  CREATE TABLE instruments (                                "
@@ -86,7 +91,7 @@ namespace LinuxSampler {
             "      UNIQUE (dir_id,instr_name)                            "
             "  );                                                        ";
         
-        GetInstrumentsDb()->ExecSql(sql);
+        ExecSql(sql);
     }
 
     InstrumentsDb::InstrumentsDb() {
@@ -95,6 +100,7 @@ namespace LinuxSampler {
     }
 
     InstrumentsDb::~InstrumentsDb() {
+        InstrumentsDbThread.StopThread();
         if (db != NULL) sqlite3_close(db);
     }
     
@@ -111,45 +117,85 @@ namespace LinuxSampler {
     }
     
     void InstrumentsDb::SetDbFile(String File) {
-        DbInstrumentsMutex.Lock();
+        LockGuard lock(DbInstrumentsMutex);
         if (File.empty() || DbFile.length() > 0) {
-            DbInstrumentsMutex.Unlock();
             throw Exception("Failed to set the database file");
         }
         DbFile = File;
-        DbInstrumentsMutex.Unlock();
+    }
+
+    String InstrumentsDb::GetDefaultDBLocation() {
+        #ifdef WIN32
+        char* userprofile = getenv("USERPROFILE");
+        if (userprofile) {
+            String s = userprofile;
+            s += "\\.linuxsampler\\instruments.db";
+            return s;
+        } else {
+            // in case USERPROFILE is not set (which should not occur)
+            return "instruments.db";
+        }
+        #else // POSIX ...
+        String s = CONFIG_DEFAULT_INSTRUMENTS_DB_LOCATION;
+        # if defined(__APPLE__)
+        if (s.find("~") == 0)
+            s.replace(0, 1, getenv("HOME"));
+        # endif
+        return s;
+        #endif
+    }
+
+    void InstrumentsDb::EnsureDBFileExists() {
+        if (DbFile.empty())
+            DbFile = GetDefaultDBLocation();
+		#if defined(__APPLE__)  /* 20071224 Toshi Nagata  */
+		if (DbFile.find("~") == 0)
+			DbFile.replace(0, 1, getenv("HOME"));
+		#endif
+        Path DbPath(DbFile);
+        String DbDir = DbPath.stripLastName();
+        // create directory if it does not exist yet
+        if (!DbPath.nodes().empty()) {
+            File d(DbDir);
+            if (!d.Exist()) {
+                #ifdef WIN32
+                if (_mkdir(DbDir.c_str()))
+                    throw Exception("Could not create instruments DB directory '" + DbDir + "'");
+                #else
+                if (mkdir(DbDir.c_str(), S_IRWXU))
+                    throw Exception("Could not create instruments DB directory '" + DbDir + "'");
+                #endif
+            }
+        }
+        // create database file if it does not exist yet
+        File f(DbFile);
+        if (!f.Exist()) {
+            // formats the DB, which creates a new instruments.db file
+            Format();
+        }
     }
 
     sqlite3* InstrumentsDb::GetDb() {
         if ( db != NULL) return db;
 
-        if (DbFile.empty()) {
-		    #ifndef WIN32
-		    DbFile = CONFIG_DEFAULT_INSTRUMENTS_DB_LOCATION;
-			#else
-			char *userprofile = getenv("USERPROFILE");
-			if(userprofile) {
-			    String DbPath = userprofile;
-				DbPath += "\\.linuxsampler";
-			    DbFile = DbPath + "\\instruments.db";
-				File InstrumentsDbFile(DbFile);
-				// if no DB exists create the subdir and then the DB
-				if( !InstrumentsDbFile.Exist() ) {
-				    _mkdir( DbPath.c_str() );
-					// formats the DB, which creates a new instruments.db file
-					Format();
-				}
-		    }
-			else {
-			    // in case USERPROFILE is not set (which should not occur)
-			    DbFile = "instruments.db";
-			}
-			#endif
-	    }
-		#if defined(__APPLE__)  /* 20071224 Toshi Nagata  */
-		if (DbFile.find("~") == 0)
-			DbFile.replace(0, 1, getenv("HOME"));
-		#endif
+        if (DbFile.empty())
+            DbFile = GetDefaultDBLocation();
+
+        {
+            // first check if the instruments DB's directory exists, if not give up
+            Path path(DbFile);
+            String sDir = path.stripLastName();
+            File d(sDir);
+            if (!d.Exist())
+                throw Exception("Instruments DB directory '" + sDir + "' does not exist!");
+
+            // just to give the user a notice about the DB file being created in case it does not exist yet
+            File f(DbFile);
+            if (!f.Exist())
+                dmsg(0,("Instruments DB file '%s' does not exist yet. Trying to create it now.\n", DbFile.c_str()));
+        }
+
+        dmsg(0,("Opening instruments DB at '%s'\n", DbFile.c_str()));
         int rc = sqlite3_open(DbFile.c_str(), &db);
         if (rc) {
             sqlite3_close(db);
@@ -199,7 +245,7 @@ namespace LinuxSampler {
             }
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
         EndTransaction();
         if (i == -1) throw Exception("Unkown DB directory: " + toEscapedPath(Dir));
@@ -236,7 +282,7 @@ namespace LinuxSampler {
             return pDirs;
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
     }
     
@@ -267,13 +313,13 @@ namespace LinuxSampler {
         }
         
         int id = 0, i = 1;
-        int j = Dir.find('/', i);
+        int j = (int) Dir.find('/', i);
 
         while(j != std::string::npos) {
             id = GetDirectoryId(id, Dir.substr(i, j - i));
             i = j + 1;
             if (i >= Dir.length()) return id;
-            j = Dir.find('/', i);
+            j = (int) Dir.find('/', i);
         }
         
         return GetDirectoryId(id, Dir.substr(i));
@@ -345,7 +391,7 @@ namespace LinuxSampler {
             }
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
         EndTransaction();
         
@@ -382,7 +428,7 @@ namespace LinuxSampler {
             ExecSql(sql.str(), toDbName(dirName));
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
 
         EndTransaction();
@@ -405,7 +451,7 @@ namespace LinuxSampler {
             RemoveDirectory(dirId);
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
 
         EndTransaction();
@@ -464,17 +510,10 @@ namespace LinuxSampler {
 
     bool InstrumentsDb::DirectoryExist(String Dir) {
         dmsg(2,("InstrumentsDb: DirectoryExist(Dir=%s)\n", Dir.c_str()));
-        bool b;
-
-        DbInstrumentsMutex.Lock();
-        try { b = GetDirectoryId(Dir) != -1; }
-        catch (Exception e) {
-            DbInstrumentsMutex.Unlock();
-            throw e;
+        {
+            LockGuard lock(DbInstrumentsMutex);
+            return GetDirectoryId(Dir) != -1;
         }
-        DbInstrumentsMutex.Unlock();
-
-        return b;
     }
 
     DbDirectory InstrumentsDb::GetDirectoryInfo(String Dir) {
@@ -515,7 +554,7 @@ namespace LinuxSampler {
             sqlite3_finalize(pStmt);
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
 
         EndTransaction();
@@ -552,7 +591,7 @@ namespace LinuxSampler {
             ExecSql(sql.str(), dbName);
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
 
         EndTransaction();
@@ -597,7 +636,7 @@ namespace LinuxSampler {
             ExecSql(sql.str());
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
 
         EndTransaction();
@@ -641,7 +680,7 @@ namespace LinuxSampler {
             DirectoryTreeWalk(Dir, &directoryCopier);
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
 
         EndTransaction();
@@ -662,7 +701,7 @@ namespace LinuxSampler {
             ExecSql(sql.str(), Desc);
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
         EndTransaction();
         
@@ -714,8 +753,9 @@ namespace LinuxSampler {
         dmsg(2,("InstrumentsDb: AddInstruments(DbDir=%s,insDir=%d,FilePath=%s,Index=%d)\n", DbDir.c_str(), insDir, FilePath.c_str(), Index));
         if (DbDir.empty() || FilePath.empty()) return;
         
-        DbInstrumentsMutex.Lock();
-        try {
+        {
+            LockGuard lock(DbInstrumentsMutex);
+
             int dirId = GetDirectoryId(DbDir);
             if (dirId == -1) throw Exception("Invalid DB directory: " + toEscapedText(DbDir));
 
@@ -734,20 +774,16 @@ namespace LinuxSampler {
 
             String dir = insDir ? PrepareSubdirectory(DbDir, FilePath) : DbDir;
             AddInstrumentsFromFile(dir, FilePath, Index, pProgress);
-        } catch (Exception e) {
-            DbInstrumentsMutex.Unlock();
-            throw e;
         }
-
-        DbInstrumentsMutex.Unlock();
     }
 
     void InstrumentsDb::AddInstrumentsNonrecursive(String DbDir, String FsDir, bool insDir, ScanProgress* pProgress) {
         dmsg(2,("InstrumentsDb: AddInstrumentsNonrecursive(DbDir=%s,FsDir=%s,insDir=%d)\n", DbDir.c_str(), FsDir.c_str(), insDir));
         if (DbDir.empty() || FsDir.empty()) return;
         
-        DbInstrumentsMutex.Lock();
-        try {
+        {
+            LockGuard lock(DbInstrumentsMutex);
+
             int dirId = GetDirectoryId(DbDir);
             if (dirId == -1) throw Exception("Invalid DB directory: " + toEscapedPath(DbDir));
 
@@ -774,15 +810,8 @@ namespace LinuxSampler {
                 }
             } catch(Exception e) {
                 e.PrintMessage();
-                DbInstrumentsMutex.Unlock();
-                return;
             }
-        } catch (Exception e) {
-            DbInstrumentsMutex.Unlock();
-            throw e;
         }
-
-        DbInstrumentsMutex.Unlock();
     }
 
     void InstrumentsDb::AddInstrumentsRecursive(String DbDir, String FsDir, bool Flat, bool insDir, ScanProgress* pProgress) {
@@ -821,7 +850,7 @@ namespace LinuxSampler {
             }
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
         EndTransaction();
 
@@ -866,7 +895,7 @@ namespace LinuxSampler {
             return pInstrs;
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
     }
 
@@ -909,7 +938,7 @@ namespace LinuxSampler {
             RemoveInstrument(instrId);
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
         EndTransaction();
         FireInstrumentCountChanged(ParentDir);
@@ -943,7 +972,7 @@ namespace LinuxSampler {
             i = GetInstrumentInfo(id);
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
         EndTransaction();
         
@@ -1018,7 +1047,7 @@ namespace LinuxSampler {
             ExecSql(sql.str(), toDbName(Name));
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
         EndTransaction();
         FireInstrumentNameChanged(Instr, toAbstractName(Name));
@@ -1061,7 +1090,7 @@ namespace LinuxSampler {
             ExecSql(sql.str());
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
         EndTransaction();
         FireInstrumentCountChanged(ParentDir);
@@ -1092,7 +1121,7 @@ namespace LinuxSampler {
             CopyInstrument(instrId, instrName, dstId, Dst);
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
         EndTransaction();
         
@@ -1157,178 +1186,149 @@ namespace LinuxSampler {
             ExecSql(sql.str(), Desc);
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
         EndTransaction();
         FireInstrumentInfoChanged(Instr);
     }
 
-    void InstrumentsDb::AddInstrumentsFromFile(String DbDir, String File, int Index, ScanProgress* pProgress) {
-        dmsg(2,("InstrumentsDb: AddInstrumentsFromFile(DbDir=%s,File=%s,Index=%d)\n", DbDir.c_str(), File.c_str(), Index));
-        
-        if(File.length() < 4) return;
-        
+    void InstrumentsDb::AddInstrumentsFromFile(String DbDir, String file, int Index, ScanProgress* pProgress) {
+        dmsg(2,("InstrumentsDb: AddInstrumentsFromFile(DbDir=%s,File=%s,Index=%d)\n", DbDir.c_str(), file.c_str(), Index));
+
+        if (!InstrumentFileInfo::isSupportedFile(file)) return;
+
         try {
-            if(!strcasecmp(".gig", File.substr(File.length() - 4).c_str())) {
-                if (pProgress != NULL) {
-                    pProgress->SetStatus(0);
-                    pProgress->CurrentFile = File;
-                }
+            if (pProgress != NULL) {
+                pProgress->SetStatus(0);
+                pProgress->CurrentFile = file;
+            }
 
-                AddGigInstruments(DbDir, File, Index, pProgress);
+            int dirId = GetDirectoryId(DbDir);
+            if (dirId == -1) throw Exception("Invalid DB directory: " + toEscapedPath(DbDir));
 
-                if (pProgress != NULL) {
-                    pProgress->SetScannedFileCount(pProgress->GetScannedFileCount() + 1);
-                }
+            File f = File(file);
+            if (!f.Exist()) {
+                std::stringstream ss;
+                ss << "Fail to stat `" << file << "`: " << f.GetErrorMsg();
+                throw Exception(ss.str());
+            }
+
+            if (!f.IsFile()) {
+                std::stringstream ss;
+                ss << "`" << file << "` is not a regular file";
+                throw Exception(ss.str());
+            }
+
+            AddInstrumentsFromFilePriv(DbDir, dirId, file, f, Index, pProgress);
+
+            if (pProgress != NULL) {
+                pProgress->SetScannedFileCount(pProgress->GetScannedFileCount() + 1);
             }
         } catch(Exception e) {
             e.PrintMessage();
         }
     }
 
-    void InstrumentsDb::AddGigInstruments(String DbDir, String FilePath, int Index, ScanProgress* pProgress) {
+    void InstrumentsDb::AddInstrumentsFromFilePriv(String DbDir, const int dirId, String FilePath, File file, int Index, ScanProgress* pProgress) {
         dmsg(2,("InstrumentsDb: AddGigInstruments(DbDir=%s,FilePath=%s,Index=%d)\n", DbDir.c_str(), FilePath.c_str(), Index));
-        int dirId = GetDirectoryId(DbDir);
-        if (dirId == -1) throw Exception("Invalid DB directory: " + toEscapedPath(DbDir));
-
-        File f = File(FilePath);
-        if (!f.Exist()) {
-            std::stringstream ss;
-            ss << "Fail to stat `" << FilePath << "`: " << f.GetErrorMsg();
-            throw Exception(ss.str());
-        }
-
-        if (!f.IsFile()) {
-            std::stringstream ss;
-            ss << "`" << FilePath << "` is not a regular file";
-            throw Exception(ss.str());
-        }
 
         bool unlocked = false;
-        RIFF::File* riff = NULL;
-        gig::File* gig = NULL;
+        InstrumentFileInfo* fileInfo = NULL;
+        sqlite3_stmt* pStmt = NULL;
         try {
-            riff = new RIFF::File(FilePath);
-            gig::File* gig = new gig::File(riff);
-            gig->SetAutoLoad(false); // avoid time consuming samples scanning
+            fileInfo = InstrumentFileInfo::getFileInfoFor(FilePath);
+            if (!fileInfo) return;
 
             std::stringstream sql;
             sql << "INSERT INTO instruments (dir_id,instr_name,instr_file,";
             sql << "instr_nr,format_family,format_version,instr_size,";
             sql << "description,is_drum,product,artists,keywords) VALUES (";
-            sql << dirId << ",?,?,?,'GIG',?," << f.GetSize() << ",?,?,?,?,?)";
+            sql << dirId << ",?,?,?,?,?," << file.GetSize() << ",?,?,?,?,?)";
 
-            sqlite3_stmt* pStmt = NULL;
+            // instr_name 1
+            // instr_file 2
+            // instr_nr 3
+            // format_family 4
+            // format_version 5
+            // description 6
+            // is_drum 7
+            // product 8
+            // artists 9
+            // keywords 10
 
             int res = sqlite3_prepare(GetDb(), sql.str().c_str(), -1, &pStmt, NULL);
             if (res != SQLITE_OK) {
                 throw Exception("DB error: " + ToString(sqlite3_errmsg(db)));
             }
 
-            String s = FilePath;
-            s = toEscapedFsPath(s);
-            BindTextParam(pStmt, 2, s);
-            String ver = "";
-            if (gig->pVersion != NULL) ver = ToString(gig->pVersion->major);
-            BindTextParam(pStmt, 4, ver);
+            BindTextParam(pStmt, 2, toEscapedFsPath(FilePath));
+            BindTextParam(pStmt, 4, fileInfo->formatName());
+            BindTextParam(pStmt, 5, fileInfo->formatVersion());
 
-            if (Index == -1) {
-                int instrIndex = 0;
-                // Assume that it's locked and should be unlocked at this point
-                // to be able to use the database from another threads
-                if (!InTransaction) {
-                    DbInstrumentsMutex.Unlock();
-                    unlocked = true;
-                } else {
-                    std::cerr << "Shouldn't be in transaction when adding instruments." << std::endl;
-                }
+            int instrIndex = (Index == -1) ? 0 : Index;
 
-                if (pProgress != NULL) gig->GetInstrument(0, &(pProgress->GigFileProgress)); // TODO: this workaround should be fixed
-                gig::Instrument* pInstrument = gig->GetFirstInstrument();
-
-                if (!InTransaction) DbInstrumentsMutex.Lock();
-                while (pInstrument) {
-                    BindTextParam(pStmt, 7, gig->pInfo->Product);
-                    BindTextParam(pStmt, 8, gig->pInfo->Artists);
-                    BindTextParam(pStmt, 9, gig->pInfo->Keywords);
-                    AddGigInstrument(pStmt, DbDir, dirId, FilePath, pInstrument, instrIndex);
-
-                    instrIndex++;
-                    pInstrument = gig->GetNextInstrument();
-                }
+            // Assume that it's locked and should be unlocked at this point
+            // to be able to use the database from another threads
+            if (!InTransaction) {
+                DbInstrumentsMutex.Unlock();
+                unlocked = true;
             } else {
-                gig::Instrument* pInstrument;
-                if (pProgress == NULL) pInstrument = gig->GetInstrument(Index);
-                else pInstrument = gig->GetInstrument(Index, &(pProgress->GigFileProgress));
-                if (pInstrument != NULL) {
-                    BindTextParam(pStmt, 7, gig->pInfo->Product);
-                    BindTextParam(pStmt, 8, gig->pInfo->Artists);
-                    BindTextParam(pStmt, 9, gig->pInfo->Keywords);
-                    AddGigInstrument(pStmt, DbDir, dirId, FilePath, pInstrument, Index);
-                }
+                std::cerr << "Shouldn't be in transaction when adding instruments." << std::endl;
             }
 
-            sqlite3_finalize(pStmt);
-            delete gig;
-            delete riff;
-        } catch (RIFF::Exception e) {
-            if (gig != NULL) delete gig;
-            if (riff != NULL) delete riff;
+            optional<InstrumentInfo> info = fileInfo->getInstrumentInfo(0, pProgress);
+            if (!InTransaction) DbInstrumentsMutex.Lock();
+            while (info) {
+                String instrumentName = info->instrumentName;
+                if (instrumentName.empty())
+                    instrumentName = Path::getBaseName(FilePath);
+                instrumentName = GetUniqueName(dirId, instrumentName);
+
+                BindTextParam(pStmt, 8, info->product);
+                BindTextParam(pStmt, 9, info->artists);
+                BindTextParam(pStmt, 10, info->keywords);
+
+                std::stringstream sql2;
+                sql2 << "SELECT COUNT(*) FROM instruments WHERE instr_file=? AND ";
+                sql2 << "instr_nr=" << instrIndex;
+                String s = toEscapedFsPath(FilePath);
+                if (ExecSqlInt(sql2.str(), s) > 0) goto next;
+
+                BindTextParam(pStmt, 1, instrumentName);
+                BindIntParam(pStmt, 3, instrIndex);
+
+                BindTextParam(pStmt, 6, info->comments);
+                BindIntParam(pStmt, 7, info->isDrum);
+
+                res = sqlite3_step(pStmt);
+                if (res != SQLITE_DONE) {
+                    throw Exception("DB error: " + ToString(sqlite3_errmsg(db)));
+                }
+                res = sqlite3_reset(pStmt);
+                FireInstrumentCountChanged(DbDir);
+
+            next:
+                if (Index != -1) break;
+
+                instrIndex++;
+                info = fileInfo->getInstrumentInfo(instrIndex, pProgress);
+            }
+        } catch (Exception e) {
+            if (pStmt) sqlite3_finalize(pStmt);
+            if (fileInfo) delete fileInfo;
             if (unlocked) DbInstrumentsMutex.Lock();
             std::stringstream ss;
-            ss << "Failed to scan `" << FilePath << "`: " << e.Message;
-            
+            ss << "Failed to scan `" << FilePath << "`: " << e.Message();
             throw Exception(ss.str());
-        } catch (Exception e) {
-            if (gig != NULL) delete gig;
-            if (riff != NULL) delete riff;
-            if (unlocked) DbInstrumentsMutex.Lock();
-            throw e;
         } catch (...) {
-            if (gig != NULL) delete gig;
-            if (riff != NULL) delete riff;
+            if (pStmt) sqlite3_finalize(pStmt);
+            if (fileInfo) delete fileInfo;
             if (unlocked) DbInstrumentsMutex.Lock();
             throw Exception("Failed to scan `" + FilePath + "`");
         }
-    }
-
-    void InstrumentsDb::AddGigInstrument(sqlite3_stmt* pStmt, String DbDir, int DirId, String File, gig::Instrument* pInstrument, int Index) {
-        dmsg(2,("InstrumentsDb: AddGigInstrument(DbDir=%s,DirId=%d,File=%s,Index=%d)\n", DbDir.c_str(), DirId, File.c_str(), Index));
-        String name = pInstrument->pInfo->Name;
-        if (name == "") return;
-        name = GetUniqueName(DirId, name);
-        
-        std::stringstream sql2;
-        sql2 << "SELECT COUNT(*) FROM instruments WHERE instr_file=? AND ";
-        sql2 << "instr_nr=" << Index;
-        String s = toEscapedFsPath(File);
-        if (ExecSqlInt(sql2.str(), s) > 0) return;
-
-        BindTextParam(pStmt, 1, name);
-        BindIntParam(pStmt, 3, Index);
-
-        BindTextParam(pStmt, 5, pInstrument->pInfo->Comments);
-        BindIntParam(pStmt, 6, pInstrument->IsDrum);
-
-        if (!pInstrument->pInfo->Product.empty()) {
-            BindTextParam(pStmt, 7, pInstrument->pInfo->Product);
-        }
-        if (!pInstrument->pInfo->Artists.empty()) {
-            BindTextParam(pStmt, 8, pInstrument->pInfo->Artists);
-        }
-
-        if (!pInstrument->pInfo->Keywords.empty()) {
-            BindTextParam(pStmt, 9, pInstrument->pInfo->Keywords);
-        }
-
-        int res = sqlite3_step(pStmt);
-        if(res != SQLITE_DONE) {
-            sqlite3_finalize(pStmt);
-            throw Exception("DB error: " + ToString(sqlite3_errmsg(db)));
-        }
-
-        res = sqlite3_reset(pStmt);
-        FireInstrumentCountChanged(DbDir);
+        if (pStmt) sqlite3_finalize(pStmt);
+        if (fileInfo) delete fileInfo;
+        if (unlocked) DbInstrumentsMutex.Lock();
     }
 
     void InstrumentsDb::DirectoryTreeWalk(String AbstractPath, DirectoryHandler* pHandler) {
@@ -1366,7 +1366,7 @@ namespace LinuxSampler {
             else directoryFinder.ProcessDirectory(Dir, DirId);
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
         EndTransaction();
 
@@ -1386,7 +1386,7 @@ namespace LinuxSampler {
             else instrumentFinder.ProcessDirectory(Dir, DirId);
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
         EndTransaction();
 
@@ -1404,12 +1404,12 @@ namespace LinuxSampler {
                 File f(toNonEscapedFsPath(files->at(i)));
                 if (!f.Exist()) result->push_back(files->at(i));
             }
+            EndTransaction();
             return result;
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
-        EndTransaction();
     }
     
     void InstrumentsDb::SetInstrumentFilePath(String OldPath, String NewPath) {
@@ -1424,7 +1424,7 @@ namespace LinuxSampler {
             ExecSql("UPDATE instruments SET instr_file=? WHERE instr_file=?", params);
         } catch (Exception e) {
             EndTransaction();
-            throw e;
+            throw; // re-throw
         }
         EndTransaction();
         
@@ -1466,7 +1466,7 @@ namespace LinuxSampler {
         }
         InTransaction = false;
         
-        if(db == NULL) {
+        if (db == NULL) {
             DbInstrumentsMutex.Unlock();
             return;
         }
@@ -1670,7 +1670,7 @@ namespace LinuxSampler {
 
     void InstrumentsDb::BindTextParam(sqlite3_stmt* pStmt, int Index, String Text) {
         if (pStmt == NULL) return;
-        int res = sqlite3_bind_text(pStmt, Index, Text.c_str(), -1, SQLITE_STATIC);
+        int res = sqlite3_bind_text(pStmt, Index, Text.c_str(), -1, SQLITE_TRANSIENT);
         if (res != SQLITE_OK) {
             sqlite3_finalize(pStmt);
             throw Exception("DB error: " + ToString(sqlite3_errmsg(db)));
@@ -1704,7 +1704,7 @@ namespace LinuxSampler {
         if (File.at(0) != '/') String("");
         if (File.length() == 1) return File;
         if (File.at(File.length() - 1) == '/') return File.substr(0, File.length() - 1);
-        int i = File.rfind('/', File.length() - 1);
+        int i = (int) File.rfind('/', File.length() - 1);
         if(i == std::string::npos) return String("");
         if(i == 0) return String("/");
         return File.substr(0, i);
@@ -1714,56 +1714,52 @@ namespace LinuxSampler {
         if (Path.length() < 2) return String("");
         if (Path.at(0) != '/') String("");
         if (Path.at(Path.length() - 1) == '/') return String("");
-        int i = Path.rfind('/', Path.length() - 1);
+        int i = (int) Path.rfind('/', Path.length() - 1);
         return Path.substr(i + 1);
     }
 
     void InstrumentsDb::CheckPathName(String Path) {
         if (Path.empty()) return;
 
-        int i = 0, j = Path.find('/', i);
+        int i = 0, j = (int)Path.find('/', i);
 
         while(j != std::string::npos) {
             if (j + 1 >= Path.length()) return;
             if (Path.at(j + 1) == '/') throw Exception("Invalid path name: " + Path);
             
             i = j + 1;
-            j = Path.find('/', i);
+            j = (int) Path.find('/', i);
         }
     }
     
     String InstrumentsDb::GetParentDirectory(String Dir) {
         if (Dir.length() < 2) return String("");
         if (Dir.at(0) != '/') String("");
-        int i = Dir.rfind('/', Dir.length() - 2);
+        int i = (int) Dir.rfind('/', Dir.length() - 2);
         if (i == 0) return "/";
         return Dir.substr(0, i);
     }
 
     void InstrumentsDb::Format() {
-        DbInstrumentsMutex.Lock();
-        if (db != NULL) {
-            sqlite3_close(db);
-            db = NULL;
-        }
+        {
+            LockGuard lock(DbInstrumentsMutex);
 
-        if (DbFile.empty()) DbFile = CONFIG_DEFAULT_INSTRUMENTS_DB_LOCATION;
-        String bkp = DbFile + ".bkp";
-        remove(bkp.c_str());
-        if (rename(DbFile.c_str(), bkp.c_str()) && errno != ENOENT) {
-            DbInstrumentsMutex.Unlock();
-            throw Exception(String("Failed to backup database: ") + strerror(errno));
-        }
+            if (db != NULL) {
+                sqlite3_close(db);
+                db = NULL;
+            }
+
+            if (DbFile.empty()) DbFile = GetDefaultDBLocation();
+            String bkp = DbFile + ".bkp";
+            remove(bkp.c_str());
+            if (rename(DbFile.c_str(), bkp.c_str()) && errno != ENOENT) {
+                throw Exception(String("Failed to backup database: ") + strerror(errno));
+            }
         
-        String f = DbFile;
-        DbFile = "";
-        try { CreateInstrumentsDb(f); }
-        catch(Exception e) {
-            DbInstrumentsMutex.Unlock();
-            throw e;
+            String f = DbFile;
+            DbFile = "";
+            CreateInstrumentsDb(f);
         }
-        DbInstrumentsMutex.Unlock();
-        
         FireDirectoryCountChanged("/");
         FireInstrumentCountChanged("/");
     }
