@@ -24,7 +24,11 @@
 #define __LS_INSTRUMENTSDB_H__
 
 #include <sqlite3.h>
-#include <gig.h>
+#if AC_APPLE_UNIVERSAL_BUILD
+# include <libgig/gig.h>
+#else
+# include <gig.h>
+#endif
 #include "../common/Mutex.h"
 #include "../common/WorkerThread.h"
 #include "../EventListeners.h"
@@ -124,10 +128,10 @@ namespace LinuxSampler {
 
             /**
              * Creates an instruments database file.
-             * @param FilePath the pathname of the file to create.
+             * @param FilePath optional pathname of the file to create.
              * @throws Exception If the creation of the database file failed.
              */
-            static void CreateInstrumentsDb(String FilePath);
+            void CreateInstrumentsDb(String FilePath = "");
 
             /**
              * This method is used to access the instruments database.
@@ -757,17 +761,8 @@ namespace LinuxSampler {
              * Specify NULL if you don't want to monitor the scanning process.
              * @throws Exception if the operation failed.
              */
-            void AddGigInstruments(String DbDir, String FilePath, int Index = -1, ScanProgress* pProgress = NULL);
+            void AddInstrumentsFromFilePriv(String DbDir, const int dirId, String FilePath, File file, int Index = -1, ScanProgress* pProgress = NULL);
 
-            /**
-             * Adds the specified GIG instrument.
-             * @param DbDir The instruments database directory
-             * in which the instrument will be added.
-             * All slashes in the directory names should be replaced with '\0'.
-             * @throws Exception if the operation failed.
-             */
-            void AddGigInstrument(sqlite3_stmt* pStmt, String DbDir, int DirId, String File, ::gig::Instrument* pInstrument, int Index);
-            
             void DirectoryTreeWalk(String AbstractPath, DirectoryHandler* pHandler);
 
             void DirectoryTreeWalk(DirectoryHandler* pHandler, String AbstractPath, int DirId, int Level);
@@ -861,6 +856,8 @@ namespace LinuxSampler {
              */
             String PrepareSubdirectory(String DbDir, String FsPath);
 
+            void EnsureDBFileExists();
+
             /**
              * Adds the specified node to the specified database directory path.
              * @returns The newly created instruments database path.
@@ -921,6 +918,8 @@ namespace LinuxSampler {
              */
             static void CheckFileName(String File);
 
+            static String GetDefaultDBLocation();
+
 #ifndef WIN32
             /** SQLite user function for handling regular expressions */
             static void Regexp(sqlite3_context* pContext, int argc, sqlite3_value** ppValue);
