@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2009 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2013 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -43,6 +43,7 @@ namespace LinuxSampler {
     // just symbol prototyping
     class Engine;
     class AudioOutputDeviceFactory;
+    class IDGenerator;
 
     /** Abstract base class for audio output drivers in LinuxSampler
      *
@@ -64,12 +65,12 @@ namespace LinuxSampler {
                 public:
                     ParameterActive();
                     ParameterActive(String s);
-                    virtual String Description();
-                    virtual bool   Fix();
-                    virtual bool   Mandatory();
-                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters();
-                    virtual optional<bool> DefaultAsBool(std::map<String,String> Parameters);
-                    virtual void OnSetValue(bool b) throw (Exception);
+                    virtual String Description() OVERRIDE;
+                    virtual bool   Fix() OVERRIDE;
+                    virtual bool   Mandatory() OVERRIDE;
+                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters() OVERRIDE;
+                    virtual optional<bool> DefaultAsBool(std::map<String,String> Parameters) OVERRIDE;
+                    virtual void OnSetValue(bool b) throw (Exception) OVERRIDE;
                     static String Name();
             };
 
@@ -81,16 +82,16 @@ namespace LinuxSampler {
                 public:
                     ParameterSampleRate();
                     ParameterSampleRate(String s);
-                    virtual String Description();
-                    virtual bool   Fix();
-                    virtual bool   Mandatory();
-                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters();
-                    virtual optional<int>    DefaultAsInt(std::map<String,String> Parameters);
-                    virtual optional<int>    RangeMinAsInt(std::map<String,String> Parameters);
-                    virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters);
-                    virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters);
-                    virtual int              ValueAsInt();
-                    virtual void             OnSetValue(int i) throw (Exception);
+                    virtual String Description() OVERRIDE;
+                    virtual bool   Fix() OVERRIDE;
+                    virtual bool   Mandatory() OVERRIDE;
+                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters() OVERRIDE;
+                    virtual optional<int>    DefaultAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual optional<int>    RangeMinAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual int              ValueAsInt() OVERRIDE;
+                    virtual void             OnSetValue(int i) throw (Exception) OVERRIDE;
                     static String Name();
             };
 
@@ -103,15 +104,15 @@ namespace LinuxSampler {
                 public:
                     ParameterChannels();
                     ParameterChannels(String s);
-                    virtual String Description();
-                    virtual bool   Fix();
-                    virtual bool   Mandatory();
-                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters();
-                    virtual optional<int>    DefaultAsInt(std::map<String,String> Parameters);
-                    virtual optional<int>    RangeMinAsInt(std::map<String,String> Parameters);
-                    virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters);
-                    virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters);
-                    virtual void             OnSetValue(int i) throw (Exception);
+                    virtual String Description() OVERRIDE;
+                    virtual bool   Fix() OVERRIDE;
+                    virtual bool   Mandatory() OVERRIDE;
+                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters() OVERRIDE;
+                    virtual optional<int>    DefaultAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual optional<int>    RangeMinAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual void             OnSetValue(int i) throw (Exception) OVERRIDE;
                     static String Name();
             };
 
@@ -180,6 +181,23 @@ namespace LinuxSampler {
              * AcquireChannels(). Each driver must implement it.
              */
             virtual AudioChannel* CreateChannel(uint ChannelNr) = 0;
+        
+            /**
+             * Might be optionally implemented by the deriving driver. If
+             * implemented, this method will return the current audio
+             * latency in seconds. The driver might measure the actual
+             * latency with timing functions to return the real latency.
+             *
+             * If this method is not implemented / overridden by the
+             * descending driver, it will simply return the theoretical
+             * latency, that is MaxSamplesPerCycle() / SampleRate().
+             *
+             * Currently this method is not used by the sampler itself.
+             * It can be used however for GUIs built on top of the sampler
+             * for example, to i.e. display the user the currently
+             * effective audio latency in real-time.
+             */
+            virtual float latency();
 
 
 
@@ -205,6 +223,20 @@ namespace LinuxSampler {
              * @param pEngine - sampler engine
              */
             void Disconnect(Engine* pEngine);
+        
+            /**
+             * This method can or should be called by the deriving driver in
+             * case some fundamental parameter changed, especially if the
+             * values returned by MaxSamplesPerCycle() and SamplerRate() have
+             * changed, those are values which the sampler engines assume to be
+             * constant for the life time of an audio device !
+             *
+             * By forcing a re-connection with this method, the sampler engines
+             * are forced to update those informations metntioned above as well,
+             * avoiding crashes or other undesired misbehaviors in such
+             * circumstances.
+             */
+            void ReconnectAll();
 
             /**
              * Returns audio channel with index \a ChannelIndex or NULL if
@@ -244,29 +276,55 @@ namespace LinuxSampler {
             std::map<String,DeviceCreationParameter*> DeviceParameters();
 
             /**
-             * Add a chain of master effects to this AudioOutputDevice.
+             * Add a chain of send effects to this AudioOutputDevice.
              * You actually have to add effects to that chain afterwards.
              */
-            EffectChain* AddMasterEffectChain();
+            EffectChain* AddSendEffectChain();
 
             /**
-             * Remove the master effect chain given by @a iChain .
+             * Remove the send effect chain given by @a iChain .
              *
-             * @throws Exception - if given master effect chain doesn't exist
+             * @throws Exception - if given send effect chain doesn't exist
              */
-            void RemoveMasterEffectChain(uint iChain) throw (Exception);
+            void RemoveSendEffectChain(uint iChain) throw (Exception);
 
             /**
-             * Returns master effect chain given by @a iChain or @c NULL if
+             * Returns send effect chain given by @a iChain or @c NULL if
              * there's no such effect chain.
              */
-            EffectChain* MasterEffectChain(uint iChain) const;
+            EffectChain* SendEffectChain(uint iChain) const;
 
             /**
-             * Returns amount of master effect chains this AudioOutputDevice
+             * Returns send effect chain with ID @a iChainID or @c NULL if
+             * there's no such effect chain.
+             */
+            EffectChain* SendEffectChainByID(uint iChainID) const;
+
+            /**
+             * Returns amount of send effect chains this AudioOutputDevice
              * currently provides.
              */
-            uint MasterEffectChainCount() const;
+            uint SendEffectChainCount() const;
+
+            /**
+             * @deprecated This method will be removed, use AddSendEffectChain() instead!
+             */
+            EffectChain* AddMasterEffectChain() DEPRECATED_API;
+
+            /**
+             * @deprecated This method will be removed, use RemoveSendEffectChain() instead!
+             */
+            void RemoveMasterEffectChain(uint iChain) throw (Exception) DEPRECATED_API;
+
+            /**
+             * @deprecated This method will be removed, use SendEffectChain() instead!
+             */
+            EffectChain* MasterEffectChain(uint iChain) const DEPRECATED_API;
+
+            /**
+             * @deprecated This method will be removed, use SendEffectChainCount() instead!
+             */
+            uint MasterEffectChainCount() const DEPRECATED_API;
 
         protected:
             SynchronizedConfig<std::set<Engine*> >    Engines;     ///< All sampler engines that are connected to the audio output device.
@@ -274,6 +332,7 @@ namespace LinuxSampler {
             std::vector<AudioChannel*>                Channels;    ///< All audio channels of the audio output device. This is just a container; the descendant has to create channels by himself.
             std::map<String,DeviceCreationParameter*> Parameters;  ///< All device parameters.
             std::vector<EffectChain*>                 vEffectChains;
+            IDGenerator*                              EffectChainIDs;
 
             AudioOutputDevice(std::map<String,DeviceCreationParameter*> DriverParameters);
 
