@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2007 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2020 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -45,14 +45,14 @@ namespace LinuxSampler {
             ~AudioOutputDeviceAlsa();
 
             // derived abstract methods from class 'AudioOutputDevice'
-            virtual void Play();
-            virtual bool IsPlaying();
-            virtual void Stop();
-            virtual uint MaxSamplesPerCycle();
-            virtual uint SampleRate();
-            virtual AudioChannel* CreateChannel(uint ChannelNr);
-
-            virtual String Driver();
+            virtual void Play() OVERRIDE;
+            virtual bool IsPlaying() OVERRIDE;
+            virtual void Stop() OVERRIDE;
+            virtual uint MaxSamplesPerCycle() OVERRIDE;
+            virtual uint SampleRate() OVERRIDE;
+            virtual AudioChannel* CreateChannel(uint ChannelNr) OVERRIDE;
+            virtual String Driver() OVERRIDE;
+            
             static String Name();
             static String Description();
             static String Version();
@@ -65,13 +65,13 @@ namespace LinuxSampler {
                 public:
                     ParameterCard();
                     ParameterCard(String s) throw (Exception);
-                    virtual String Description();
-                    virtual bool   Fix();
-                    virtual bool   Mandatory();
-                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters();
-                    virtual optional<String>    DefaultAsString(std::map<String,String> Parameters);
-                    virtual std::vector<String> PossibilitiesAsString(std::map<String,String> Parameters);
-                    virtual void                OnSetValue(String s) throw (Exception);
+                    virtual String Description() OVERRIDE;
+                    virtual bool   Fix() OVERRIDE;
+                    virtual bool   Mandatory() OVERRIDE;
+                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters() OVERRIDE;
+                    virtual optional<String>    DefaultAsString(std::map<String,String> Parameters) OVERRIDE;
+                    virtual std::vector<String> PossibilitiesAsString(std::map<String,String> Parameters) OVERRIDE;
+                    virtual void                OnSetValue(String s) throw (Exception) OVERRIDE;
                     static String Name();
             };
 
@@ -83,10 +83,10 @@ namespace LinuxSampler {
                 public:
                     ParameterSampleRate();
                     ParameterSampleRate(String s);
-                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters();
-                    virtual optional<int> DefaultAsInt(std::map<String,String> Parameters);
-                    virtual optional<int> RangeMinAsInt(std::map<String,String> Parameters);
-                    virtual optional<int> RangeMaxAsInt(std::map<String,String> Parameters);
+                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters() OVERRIDE;
+                    virtual optional<int> DefaultAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual optional<int> RangeMinAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual optional<int> RangeMaxAsInt(std::map<String,String> Parameters) OVERRIDE;
             };
 
             /** Device Parameters 'CHANNELS'
@@ -98,10 +98,10 @@ namespace LinuxSampler {
                 public:
                     ParameterChannels();
                     ParameterChannels(String s);
-                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters();
-                    virtual optional<int> DefaultAsInt(std::map<String,String> Parameters);
-                    virtual optional<int> RangeMinAsInt(std::map<String,String> Parameters);
-                    virtual optional<int> RangeMaxAsInt(std::map<String,String> Parameters);
+                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters() OVERRIDE;
+                    virtual optional<int> DefaultAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual optional<int> RangeMinAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual optional<int> RangeMaxAsInt(std::map<String,String> Parameters) OVERRIDE;
             };
 
             /** Device Parameter 'FRAGMENTS'
@@ -112,15 +112,15 @@ namespace LinuxSampler {
                 public:
                     ParameterFragments();
                     ParameterFragments(String s) throw (Exception);
-                    virtual String Description();
-                    virtual bool   Fix();
-                    virtual bool   Mandatory();
-                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters();
-                    virtual optional<int>    DefaultAsInt(std::map<String,String> Parameters);
-                    virtual optional<int>    RangeMinAsInt(std::map<String,String> Parameters);
-                    virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters);
-                    virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters);
-                    virtual void             OnSetValue(int i) throw (Exception);
+                    virtual String Description() OVERRIDE;
+                    virtual bool   Fix() OVERRIDE;
+                    virtual bool   Mandatory() OVERRIDE;
+                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters() OVERRIDE;
+                    virtual optional<int>    DefaultAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual optional<int>    RangeMinAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual void             OnSetValue(int i) throw (Exception) OVERRIDE;
                     static String Name();
             };
 
@@ -132,20 +132,20 @@ namespace LinuxSampler {
                 public:
                     ParameterFragmentSize();
                     ParameterFragmentSize(String s) throw (Exception);
-                    virtual String Description();
-                    virtual bool   Fix();
-                    virtual bool   Mandatory();
-                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters();
-                    virtual optional<int>    DefaultAsInt(std::map<String,String> Parameters);
-                    virtual optional<int>    RangeMinAsInt(std::map<String,String> Parameters);
-                    virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters);
-                    virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters);
-                    virtual void             OnSetValue(int i) throw (Exception);
+                    virtual String Description() OVERRIDE;
+                    virtual bool   Fix() OVERRIDE;
+                    virtual bool   Mandatory() OVERRIDE;
+                    virtual std::map<String,DeviceCreationParameter*> DependsAsParameters() OVERRIDE;
+                    virtual optional<int>    DefaultAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual optional<int>    RangeMinAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters) OVERRIDE;
+                    virtual void             OnSetValue(int i) throw (Exception) OVERRIDE;
                     static String Name();
             };
 
         protected:
-            int Main();  ///< Implementation of virtual method from class Thread
+            int Main() OVERRIDE;  ///< Implementation of virtual method from class Thread
 
         private:
             uint                 uiAlsaChannels;
