@@ -3,6 +3,8 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2009 Grigor Iliev                                       *
+ *   Copyright (C) 2011-2013 Andreas Persson                               *
+ *   Copyright (C) 2014-2020 Christian Schoenebeck                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -141,11 +143,14 @@ namespace LinuxSampler {
 	}
     }
 
-    AudioOutputDeviceCoreAudio::~AudioOutputDevice() {
+    AudioOutputDeviceCoreAudio::~AudioOutputDeviceCoreAudio() {
+        StopThread();
+
         atomic_set(&(aqPlayerState.mIsRunning), 0);
-        destroyMutex.Lock();
-        AudioQueueDispose(aqPlayerState.mQueue, true);
-        destroyMutex.Unlock();
+        {
+            LockGuard lock(destroyMutex);
+            AudioQueueDispose(aqPlayerState.mQueue, true);
+        }
         delete [] aqPlayerState.mBuffers;
 
         CurrentDevice.RemoveListener(this);
@@ -160,7 +165,7 @@ namespace LinuxSampler {
     }
 
     String AudioOutputDeviceCoreAudio::Version() {
-       String s = "$Revision: 1.5 $";
+       String s = "$Revision: 3766 $";
        return s.substr(11, s.size() - 13); // cut dollar signs, spaces and CVS macro keyword
     }
 
@@ -304,6 +309,11 @@ namespace LinuxSampler {
      */
     int AudioOutputDeviceCoreAudio::Main() {
         dmsg(1,("CoreAudio thread started\n"));
+
+        #if DEBUG
+        Thread::setNameOfCaller("CoreAudio");
+        #endif
+
         OSStatus res;
         if(aqPlayerState.mQueue == NULL) {
             /*
@@ -317,36 +327,33 @@ namespace LinuxSampler {
             }
         }
 
-        destroyMutex.Lock();
-        do {
-            if(atomic_read(&pausedNew) != pausedOld) {
-                pausedOld = atomic_read(&pausedNew);
+        {
+            LockGuard lock(destroyMutex);
+            do {
+                if(atomic_read(&pausedNew) != pausedOld) {
+                    pausedOld = atomic_read(&pausedNew);
 
-                if(pausedOld) {
-                    res = AudioQueuePause(aqPlayerState.mQueue);
-                    if(res) std::cerr << "AudioQueuePause: Error " << res << std::endl;
-                } else {
-                    res = AudioQueuePrime(aqPlayerState.mQueue, 0, NULL);
-                    if(res) std::cerr << "AudioQueuePrime: Error " << res << std::endl;
-                    res = AudioQueueStart(aqPlayerState.mQueue, NULL);
-                    if(res) std::cerr << "AudioQueueStart: Error " << res << std::endl;
+                    if(pausedOld) {
+                        res = AudioQueuePause(aqPlayerState.mQueue);
+                        if(res) std::cerr << "AudioQueuePause: Error " << res << std::endl;
+                    } else {
+                        res = AudioQueuePrime(aqPlayerState.mQueue, 0, NULL);
+                        if(res) std::cerr << "AudioQueuePrime: Error " << res << std::endl;
+                        res = AudioQueueStart(aqPlayerState.mQueue, NULL);
+                        if(res) std::cerr << "AudioQueueStart: Error " << res << std::endl;
+                    }
                 }
-            }
 
-            if(atomic_read(&restartQueue)) {
-                DestroyAudioQueue();
-                try { CreateAndStartAudioQueue(); }
-                catch(Exception e) {
-                    destroyMutex.Unlock();
-                    throw e;
+                if(atomic_read(&restartQueue)) {
+                    DestroyAudioQueue();
+                    CreateAndStartAudioQueue();
+                    atomic_set(&restartQueue, 0);
+                    dmsg(1,("Audio queue restarted"));
                 }
-                atomic_set(&restartQueue, 0);
-                dmsg(1,("Audio queue restarted"));
-            }
             
-            CFRunLoopRunInMode(kCFRunLoopDefaultMode, 0.2, false);
-        } while (atomic_read(&(aqPlayerState.mIsRunning)));
-        destroyMutex.Unlock();
+                CFRunLoopRunInMode(kCFRunLoopDefaultMode, 0.2, false);
+            } while (atomic_read(&(aqPlayerState.mIsRunning)));
+        }
 
         dmsg(2,("CoreAudio thread stopped\n"));
 
@@ -425,7 +432,7 @@ namespace LinuxSampler {
     int AudioOutputDeviceCoreAudio::ParameterDevice::GetDeviceIndex() {
         String s = ValueAsString();
         if(s.empty()) return -1;
-        int n = s.find(' ');
+        int n = (int) s.find(' ');
         s = s.substr(0, n);
         return ToInt(s) - 1;
     }
