@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2008 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2020 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -28,6 +28,9 @@
 
 namespace LinuxSampler {
 
+    /// number of currently existing ALSA midi input devices in LinuxSampler
+    static int existingAlsaDevices = 0;
+
 // *************** ParameterName ***************
 // *
 
@@ -119,6 +122,50 @@ namespace LinuxSampler {
 
 
 
+// *************** MidiInputDeviceAlsa::ParameterName ***************
+// *
+
+    MidiInputDeviceAlsa::ParameterName::ParameterName() : DeviceCreationParameterString() {
+        InitWithDefault(); // use default name
+    }
+
+    MidiInputDeviceAlsa::ParameterName::ParameterName(String s) : DeviceCreationParameterString(s) {
+    }
+
+    String MidiInputDeviceAlsa::ParameterName::Description() {
+        return "Arbitrary ALSA client name";
+    }
+
+    bool MidiInputDeviceAlsa::ParameterName::Fix() {
+        return true;
+    }
+
+    bool MidiInputDeviceAlsa::ParameterName::Mandatory() {
+        return false;
+    }
+
+    std::map<String,DeviceCreationParameter*> MidiInputDeviceAlsa::ParameterName::DependsAsParameters() {
+        return std::map<String,DeviceCreationParameter*>(); // no dependencies
+    }
+
+    std::vector<String> MidiInputDeviceAlsa::ParameterName::PossibilitiesAsString(std::map<String,String> Parameters) {
+        return std::vector<String>();
+    }
+
+    optional<String> MidiInputDeviceAlsa::ParameterName::DefaultAsString(std::map<String,String> Parameters) {
+        return (existingAlsaDevices) ? "LinuxSampler" + ToString(existingAlsaDevices) : "LinuxSampler";
+    }
+
+    void MidiInputDeviceAlsa::ParameterName::OnSetValue(String s) throw (Exception) {
+        // not possible, as parameter is fix
+    }
+
+    String MidiInputDeviceAlsa::ParameterName::Name() {
+        return "NAME";
+    }
+
+
+
 // *************** MidiInputPortAlsa ***************
 // *
 
@@ -192,12 +239,13 @@ namespace LinuxSampler {
         if (snd_seq_open(&hAlsaSeq, "default", SND_SEQ_OPEN_INPUT, 0) < 0) {
             throw MidiInputException("Error opening ALSA sequencer");
         }
+        existingAlsaDevices++;
         this->hAlsaSeqClient = snd_seq_client_id(hAlsaSeq);
-        snd_seq_set_client_name(hAlsaSeq, "LinuxSampler");
-	AcquirePorts(((DeviceCreationParameterInt*)Parameters["PORTS"])->ValueAsInt());
-	if (((DeviceCreationParameterBool*)Parameters["ACTIVE"])->ValueAsBool()) {
-		Listen();
-	}
+        snd_seq_set_client_name(hAlsaSeq, ((DeviceCreationParameterString*)Parameters["NAME"])->ValueAsString().c_str());
+        AcquirePorts(((DeviceCreationParameterInt*)Parameters["PORTS"])->ValueAsInt());
+        if (((DeviceCreationParameterBool*)Parameters["ACTIVE"])->ValueAsBool()) {
+            Listen();
+        }
     }
 
     MidiInputDeviceAlsa::~MidiInputDeviceAlsa() {
@@ -210,6 +258,7 @@ namespace LinuxSampler {
         Ports.clear();
 
         snd_seq_close(hAlsaSeq);
+        existingAlsaDevices--; //FIXME: this is too simple, can lead to multiple clients with the same name
     }
 
     MidiInputDeviceAlsa::MidiInputPortAlsa* MidiInputDeviceAlsa::CreateMidiPort() {
@@ -237,11 +286,16 @@ namespace LinuxSampler {
     }
 
     String MidiInputDeviceAlsa::Version() {
-	    String s = "$Revision: 1.23 $";
+	    String s = "$Revision: 3766 $";
 	    return s.substr(11, s.size() - 13); // cut dollar signs, spaces and CVS macro keyword
     }
 
     int MidiInputDeviceAlsa::Main() {
+
+        #if DEBUG
+        Thread::setNameOfCaller("AlsaMidi");
+        #endif
+
         int npfd;
         struct pollfd* pfd;
         snd_seq_event_t* ev;
@@ -250,8 +304,15 @@ namespace LinuxSampler {
         pfd = (struct pollfd*) alloca(npfd * sizeof(struct pollfd));
         snd_seq_poll_descriptors(hAlsaSeq, pfd, npfd, POLLIN);
         while (true) {
+            // poll() is defined as thread cancelation point by POSIX
             if (poll(pfd, npfd, 100000) > 0) {
                 do {
+                    TestCancel();
+
+                    // prevent thread from being cancelled
+                    // (e.g. to prevent deadlocks while holding mutex lock(s))
+                    pushCancelable(false);
+
                     snd_seq_event_input(hAlsaSeq, &ev);
                     int port = (int) ev->dest.port;
                     MidiInputPort* pMidiInputPort = Ports[port];
@@ -266,7 +327,11 @@ namespace LinuxSampler {
                             break;
 
                         case SND_SEQ_EVENT_CHANPRESS:
-                            pMidiInputPort->DispatchControlChange(128, ev->data.control.value, ev->data.control.channel);
+                            pMidiInputPort->DispatchChannelPressure(ev->data.control.value, ev->data.control.channel);
+                            break;
+
+                        case SND_SEQ_EVENT_KEYPRESS:
+                            pMidiInputPort->DispatchPolyphonicKeyPressure(ev->data.note.note, ev->data.note.velocity, ev->data.control.channel);
                             break;
 
                         case SND_SEQ_EVENT_PITCHBEND:
@@ -295,6 +360,11 @@ namespace LinuxSampler {
                             break;
                     }
                     snd_seq_free_event(ev);
+
+                    // now allow thread being cancelled again
+                    // (since all mutexes are now unlocked)
+                    popCancelable();
+
                 } while (snd_seq_event_input_pending(hAlsaSeq, 0) > 0);
             }
         }
