@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2009 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2016 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -77,9 +77,12 @@ namespace LinuxSampler {
     MidiInputPort::MidiInputPort(MidiInputDevice* pDevice, int portNumber)
         : MidiChannelMapReader(MidiChannelMap),
           SysexListenersReader(SysexListeners),
-          virtualMidiDevicesReader(virtualMidiDevices) {
+          virtualMidiDevicesReader(virtualMidiDevices),
+          noteOnVelocityFilterReader(noteOnVelocityFilter)
+    {
         this->pDevice = pDevice;
         this->portNumber = portNumber;
+        runningStatusBuf[0] = 0;
         Parameters["NAME"] = new ParameterName(this);
     }
 
@@ -97,18 +100,24 @@ namespace LinuxSampler {
 
     void MidiInputPort::DispatchNoteOn(uint8_t Key, uint8_t Velocity, uint MidiChannel) {
         if (Key > 127 || Velocity > 127 || MidiChannel > 16) return;
+        
+        // apply velocity filter (if any)
+        const std::vector<uint8_t>& velocityFilter = noteOnVelocityFilterReader.Lock();
+        if (!velocityFilter.empty()) Velocity = velocityFilter[Velocity];
+        noteOnVelocityFilterReader.Unlock();
+        
         const MidiChannelMap_t& midiChannelMap = MidiChannelMapReader.Lock();
         // dispatch event for engines listening to the same MIDI channel
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOn(Key, Velocity);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOn(Key, Velocity, MidiChannel);
         }
         // dispatch event for engines listening to ALL MIDI channels
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOn(Key, Velocity);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOn(Key, Velocity, MidiChannel);
         }
         MidiChannelMapReader.Unlock();
 
@@ -122,18 +131,24 @@ namespace LinuxSampler {
 
     void MidiInputPort::DispatchNoteOn(uint8_t Key, uint8_t Velocity, uint MidiChannel, int32_t FragmentPos) {
         if (Key > 127 || Velocity > 127 || MidiChannel > 16) return;
+        
+        // apply velocity filter (if any)
+        const std::vector<uint8_t>& velocityFilter = noteOnVelocityFilterReader.Lock();
+        if (!velocityFilter.empty()) Velocity = velocityFilter[Velocity];
+        noteOnVelocityFilterReader.Unlock();
+        
         const MidiChannelMap_t& midiChannelMap = MidiChannelMapReader.Lock();
         // dispatch event for engines listening to the same MIDI channel
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOn(Key, Velocity, FragmentPos);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOn(Key, Velocity, MidiChannel, FragmentPos);
         }
         // dispatch event for engines listening to ALL MIDI channels
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOn(Key, Velocity, FragmentPos);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOn(Key, Velocity, MidiChannel, FragmentPos);
         }
         MidiChannelMapReader.Unlock();
 
@@ -152,13 +167,13 @@ namespace LinuxSampler {
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOff(Key, Velocity);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOff(Key, Velocity, MidiChannel);
         }
         // dispatch event for engines listening to ALL MIDI channels
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOff(Key, Velocity);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOff(Key, Velocity, MidiChannel);
         }
         MidiChannelMapReader.Unlock();
 
@@ -177,13 +192,13 @@ namespace LinuxSampler {
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOff(Key, Velocity, FragmentPos);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOff(Key, Velocity, MidiChannel, FragmentPos);
         }
         // dispatch event for engines listening to ALL MIDI channels
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOff(Key, Velocity, FragmentPos);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendNoteOff(Key, Velocity, MidiChannel, FragmentPos);
         }
         MidiChannelMapReader.Unlock();
 
@@ -202,13 +217,13 @@ namespace LinuxSampler {
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendPitchbend(Pitch);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendPitchbend(Pitch, MidiChannel);
         }
         // dispatch event for engines listening to ALL MIDI channels
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendPitchbend(Pitch);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendPitchbend(Pitch, MidiChannel);
         }
         MidiChannelMapReader.Unlock();
     }
@@ -220,13 +235,85 @@ namespace LinuxSampler {
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendPitchbend(Pitch, FragmentPos);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendPitchbend(Pitch, MidiChannel, FragmentPos);
         }
         // dispatch event for engines listening to ALL MIDI channels
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendPitchbend(Pitch, FragmentPos);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendPitchbend(Pitch, MidiChannel, FragmentPos);
+        }
+        MidiChannelMapReader.Unlock();
+    }
+
+    void MidiInputPort::DispatchChannelPressure(uint8_t Value, uint MidiChannel) {
+        if (Value > 127 || MidiChannel > 16) return;
+        const MidiChannelMap_t& midiChannelMap = MidiChannelMapReader.Lock();
+        // dispatch event for engines listening to the same MIDI channel
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
+            for (; engineiter != end; engineiter++) (*engineiter)->SendChannelPressure(Value, MidiChannel);
+        }
+        // dispatch event for engines listening to ALL MIDI channels
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
+            for (; engineiter != end; engineiter++) (*engineiter)->SendChannelPressure(Value, MidiChannel);
+        }
+        MidiChannelMapReader.Unlock();
+    }
+
+    void MidiInputPort::DispatchChannelPressure(uint8_t Value, uint MidiChannel, int32_t FragmentPos) {
+        if (Value > 127 || MidiChannel > 16) return;
+        const MidiChannelMap_t& midiChannelMap = MidiChannelMapReader.Lock();
+        // dispatch event for engines listening to the same MIDI channel
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
+            for (; engineiter != end; engineiter++) (*engineiter)->SendChannelPressure(Value, MidiChannel, FragmentPos);
+        }
+        // dispatch event for engines listening to ALL MIDI channels
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
+            for (; engineiter != end; engineiter++) (*engineiter)->SendChannelPressure(Value, MidiChannel, FragmentPos);
+        }
+        MidiChannelMapReader.Unlock();
+    }
+
+    void MidiInputPort::DispatchPolyphonicKeyPressure(uint8_t Key, uint8_t Value, uint MidiChannel) {
+        if (Key > 127 || Value > 127 || MidiChannel > 16) return;
+        const MidiChannelMap_t& midiChannelMap = MidiChannelMapReader.Lock();
+        // dispatch event for engines listening to the same MIDI channel
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
+            for (; engineiter != end; engineiter++) (*engineiter)->SendPolyphonicKeyPressure(Key, Value, MidiChannel);
+        }
+        // dispatch event for engines listening to ALL MIDI channels
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
+            for (; engineiter != end; engineiter++) (*engineiter)->SendPolyphonicKeyPressure(Key, Value, MidiChannel);
+        }
+        MidiChannelMapReader.Unlock();
+    }
+
+    void MidiInputPort::DispatchPolyphonicKeyPressure(uint8_t Key, uint8_t Value, uint MidiChannel, int32_t FragmentPos) {
+        if (Key > 127 || Value > 127 || MidiChannel > 16) return;
+        const MidiChannelMap_t& midiChannelMap = MidiChannelMapReader.Lock();
+        // dispatch event for engines listening to the same MIDI channel
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
+            for (; engineiter != end; engineiter++) (*engineiter)->SendPolyphonicKeyPressure(Key, Value, MidiChannel, FragmentPos);
+        }
+        // dispatch event for engines listening to ALL MIDI channels
+        {
+            std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
+            std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
+            for (; engineiter != end; engineiter++) (*engineiter)->SendPolyphonicKeyPressure(Key, Value, MidiChannel, FragmentPos);
         }
         MidiChannelMapReader.Unlock();
     }
@@ -238,15 +325,22 @@ namespace LinuxSampler {
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendControlChange(Controller, Value);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendControlChange(Controller, Value, MidiChannel);
         }
         // dispatch event for engines listening to ALL MIDI channels
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendControlChange(Controller, Value);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendControlChange(Controller, Value, MidiChannel);
         }
         MidiChannelMapReader.Unlock();
+        
+        // dispatch event to all low priority MIDI listeners
+        const std::vector<VirtualMidiDevice*>& listeners =
+            virtualMidiDevicesReader.Lock();
+        for (int i = 0; i < listeners.size(); ++i)
+            listeners[i]->SendCCToDevice(Controller, Value);
+        virtualMidiDevicesReader.Unlock();
     }
 
     void MidiInputPort::DispatchControlChange(uint8_t Controller, uint8_t Value, uint MidiChannel, int32_t FragmentPos) {
@@ -256,15 +350,22 @@ namespace LinuxSampler {
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[MidiChannel].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[MidiChannel].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendControlChange(Controller, Value, FragmentPos);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendControlChange(Controller, Value, MidiChannel, FragmentPos);
         }
         // dispatch event for engines listening to ALL MIDI channels
         {
             std::set<EngineChannel*>::iterator engineiter = midiChannelMap[midi_chan_all].begin();
             std::set<EngineChannel*>::iterator end        = midiChannelMap[midi_chan_all].end();
-            for (; engineiter != end; engineiter++) (*engineiter)->SendControlChange(Controller, Value, FragmentPos);
+            for (; engineiter != end; engineiter++) (*engineiter)->SendControlChange(Controller, Value, MidiChannel, FragmentPos);
         }
         MidiChannelMapReader.Unlock();
+        
+        // dispatch event to all low priority MIDI listeners
+        const std::vector<VirtualMidiDevice*>& listeners =
+            virtualMidiDevicesReader.Lock();
+        for (int i = 0; i < listeners.size(); ++i)
+            listeners[i]->SendCCToDevice(Controller, Value);
+        virtualMidiDevicesReader.Unlock();
     }
 
     void MidiInputPort::DispatchSysex(void* pData, uint Size) {
@@ -349,8 +450,46 @@ namespace LinuxSampler {
         }
         MidiChannelMapReader.Unlock();
     }
+    
+    /**
+     * Handles the so called MIDI "running status" mode, which allows devices
+     * to reduce bandwidth (data reduction).
+     *
+     * If the passed in MIDI data is regular MIDI data, this method will simply
+     * return the original data pointer and just stores the status byte for
+     * potential "running status" event eventually coming next.
+     *
+     * If the passed in MIDI data however seems to be in "running status" mode,
+     * this method will return another buffer, which allows the MIDI parser
+     * to handle the MIDI data as usually with "normal" MIDI data.
+     */
+    uint8_t* MidiInputPort::handleRunningStatus(uint8_t* pData) {
+        if ((pData[0] & 0x80) || !runningStatusBuf[0]) {
+            // store status byte for eventual "running status" in next event
+            if (pData[0] & 0x80) {
+                if (pData[0] < 0xf0) {
+                    // "running status" is only allowed for channel messages
+                    runningStatusBuf[0] = pData[0];
+                } else if (pData[0] < 0xf8) {
+                    // "system common" messages (0xf0..0xf7) shall reset any running
+                    // status, however "realtime" messages (0xf8..0xff) shall be
+                    // ignored here
+                    runningStatusBuf[0] = 0;
+                }
+            }
+            // it's either a regular status byte, or some invalid "running status"
+            return pData;
+        } else { // "running status" mode ...
+            const uint8_t type = runningStatusBuf[0] & 0xf0;
+            const int size = (type == 0xc0 || type == 0xd0) ? 1 : 2; // only program change & channel pressure have 1 data bytes
+            memcpy(&runningStatusBuf[1], pData, size);
+            return runningStatusBuf;
+        }
+    }
 
     void MidiInputPort::DispatchRaw(uint8_t* pData) {
+        pData = handleRunningStatus(pData);
+        
         uint8_t channel = pData[0] & 0x0f;
         switch (pData[0] & 0xf0) {
         case 0x80:
@@ -363,6 +502,9 @@ namespace LinuxSampler {
                 DispatchNoteOff(pData[1], pData[2], channel);
             }
             break;
+        case 0xA0:
+            DispatchPolyphonicKeyPressure(pData[1], pData[2], channel);
+            break;
         case 0xb0:
             if (pData[1] == 0) {
                 DispatchBankSelectMsb(pData[2], channel);
@@ -375,7 +517,7 @@ namespace LinuxSampler {
             DispatchProgramChange(pData[1], channel);
             break;
         case 0xd0:
-            DispatchControlChange(128, pData[1], channel);
+            DispatchChannelPressure(pData[1], channel);
             break;
         case 0xe0:
             DispatchPitchbend((pData[1] | pData[2] << 7) - 8192, channel);
@@ -384,6 +526,8 @@ namespace LinuxSampler {
     }
 
     void MidiInputPort::DispatchRaw(uint8_t* pData, int32_t FragmentPos) {
+        pData = handleRunningStatus(pData);
+        
         uint8_t channel = pData[0] & 0x0f;
         switch (pData[0] & 0xf0) {
         case 0x80:
@@ -396,6 +540,9 @@ namespace LinuxSampler {
                 DispatchNoteOff(pData[1], pData[2], channel, FragmentPos);
             }
             break;
+        case 0xA0:
+            DispatchPolyphonicKeyPressure(pData[1], pData[2], channel, FragmentPos);
+            break;
         case 0xb0:
             if (pData[1] == 0) {
                 DispatchBankSelectMsb(pData[2], channel);
@@ -408,36 +555,64 @@ namespace LinuxSampler {
             DispatchProgramChange(pData[1], channel);
             break;
         case 0xd0:
-            DispatchControlChange(128, pData[1], channel, FragmentPos);
+            DispatchChannelPressure(pData[1], channel, FragmentPos);
             break;
         case 0xe0:
             DispatchPitchbend((pData[1] | pData[2] << 7) - 8192, channel, FragmentPos);
             break;
         }
     }
+    
+    void MidiInputPort::SetNoteOnVelocityFilter(const std::vector<uint8_t>& filter) {
+        if (filter.size() != 128 && filter.size() != 0)
+            throw MidiInputException("Note on velocity filter must be either of size 128 or 0");
+        
+        // check the value range of the filter
+        if (!filter.empty())
+            for (int i = 0; i < 128; i++)
+                if (filter[i] > 127)
+                    throw MidiInputException("Invalid note on velocity filter, values must be in range 0 .. 127");
+        
+        // apply new filter ...
+        LockGuard lock(noteOnVelocityFilterMutex);
+        // double buffer ... double work ...
+        {
+            std::vector<uint8_t>& config =
+                noteOnVelocityFilter.GetConfigForUpdate();
+            config = filter;
+        }
+        {
+            std::vector<uint8_t>& config =
+                noteOnVelocityFilter.SwitchConfig();
+            config = filter;
+        }
+    }
 
     void MidiInputPort::Connect(EngineChannel* pEngineChannel, midi_chan_t MidiChannel) {
         if (MidiChannel < 0 || MidiChannel > 16)
             throw MidiInputException("MIDI channel index out of bounds");
 
         // first check if desired connection is already established
-        MidiChannelMapMutex.Lock();
-        MidiChannelMap_t& midiChannelMap = MidiChannelMap.GetConfigForUpdate();
-        bool bAlreadyDone = midiChannelMap[MidiChannel].count(pEngineChannel);
-        MidiChannelMapMutex.Unlock();
-        if (bAlreadyDone) return;
+        {
+            LockGuard lock(MidiChannelMapMutex);
+            MidiChannelMap_t& midiChannelMap = MidiChannelMap.GetConfigForUpdate();
+            if (midiChannelMap[MidiChannel].count(pEngineChannel)) return;
+        }
 
         // remove all other connections of that engine channel (if any)
         Disconnect(pEngineChannel);
 
         // register engine channel on the desired MIDI channel
-        MidiChannelMapMutex.Lock();
-        MidiChannelMap.GetConfigForUpdate()[MidiChannel].insert(pEngineChannel);
-        MidiChannelMap.SwitchConfig()[MidiChannel].insert(pEngineChannel);
-        MidiChannelMapMutex.Unlock();
+        {
+            LockGuard lock(MidiChannelMapMutex);
+            MidiChannelMap.GetConfigForUpdate()[MidiChannel].insert(pEngineChannel);
+            MidiChannelMap.SwitchConfig()[MidiChannel].insert(pEngineChannel);
+        }
 
         // inform engine channel about this connection
-        pEngineChannel->Connect(this, MidiChannel);
+        pEngineChannel->Connect(this);
+        if (pEngineChannel->MidiChannel() != MidiChannel)
+            pEngineChannel->SetMidiChannel(MidiChannel);
 
         // mark engine channel as changed
         pEngineChannel->StatusChanged(true);
@@ -449,8 +624,8 @@ namespace LinuxSampler {
         bool bChannelFound = false;
 
         // unregister engine channel from all MIDI channels
-        MidiChannelMapMutex.Lock();
         try {
+            LockGuard lock(MidiChannelMapMutex);
             {
                 MidiChannelMap_t& midiChannelMap = MidiChannelMap.GetConfigForUpdate();
                 for (int i = 0; i <= 16; i++) {
@@ -468,10 +643,9 @@ namespace LinuxSampler {
             }
         }
         catch(...) { /* NOOP */ }
-        MidiChannelMapMutex.Unlock();
 
         // inform engine channel about the disconnection (if there is one)
-        if (bChannelFound) pEngineChannel->DisconnectMidiInputPort();
+        if (bChannelFound) pEngineChannel->Disconnect(this);
 
         // mark engine channel as changed
         pEngineChannel->StatusChanged(true);
@@ -485,13 +659,13 @@ namespace LinuxSampler {
     }
 
     bool MidiInputPort::RemoveSysexListener(Engine* engine) {
-        int count = SysexListeners.GetConfigForUpdate().erase(engine);
+        size_t count = SysexListeners.GetConfigForUpdate().erase(engine);
         if (count) SysexListeners.SwitchConfig().erase(engine);
         return count;
     }
 
     void MidiInputPort::Connect(VirtualMidiDevice* pDevice) {
-        virtualMidiDevicesMutex.Lock();
+        LockGuard lock(virtualMidiDevicesMutex);
         // double buffer ... double work ...
         {
             std::vector<VirtualMidiDevice*>& devices =
@@ -503,11 +677,10 @@ namespace LinuxSampler {
                 virtualMidiDevices.SwitchConfig();
             devices.push_back(pDevice);
         }
-        virtualMidiDevicesMutex.Unlock();
     }
 
     void MidiInputPort::Disconnect(VirtualMidiDevice* pDevice) {
-        virtualMidiDevicesMutex.Lock();
+        LockGuard lock(virtualMidiDevicesMutex);
         // double buffer ... double work ...
         {
             std::vector<VirtualMidiDevice*>& devices =
@@ -519,7 +692,23 @@ namespace LinuxSampler {
                 virtualMidiDevices.SwitchConfig();
             devices.erase(std::find(devices.begin(), devices.end(), pDevice));
         }
-        virtualMidiDevicesMutex.Unlock();
+    }
+
+    int MidiInputPort::expectedEventSize(unsigned char byte) {
+        if (!(byte & 0x80) && runningStatusBuf[0])
+            byte = runningStatusBuf[0]; // "running status" mode
+        
+        if (byte < 0x80) return -1; // not a valid status byte
+        if (byte < 0xC0) return 3; // note on/off, note pressure, control change
+        if (byte < 0xE0) return 2; // program change, channel pressure
+        if (byte < 0xF0) return 3; // pitch wheel
+        if (byte == 0xF0) return -1; // sysex message (variable size)
+        if (byte == 0xF1) return 2; // time code per quarter frame
+        if (byte == 0xF2) return 3; // sys. common song position pointer
+        if (byte == 0xF3) return 2; // sys. common song select
+        if (byte == 0xF4) return -1; // sys. common undefined / reserved
+        if (byte == 0xF5) return -1; // sys. common undefined / reserved
+        return 1; // tune request, end of SysEx, system real-time events
     }
 
 } // namespace LinuxSampler
