@@ -1,6 +1,6 @@
 /***************************************************************************
  *                                                                         *
- *   Copyright (C) 2005 - 2009 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2020 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -58,45 +58,70 @@ namespace LinuxSampler {
             // abstract methods
             //     (these have to be implemented by the descendant)
 
-            virtual void    PrepareLoadInstrument(const char* FileName, uint Instrument) = 0;
-            virtual void    LoadInstrument() = 0;
+            // general sampler part management
             virtual void    Reset() = 0;
-            virtual void    SendNoteOn(uint8_t Key, uint8_t Velocity) = 0;
-            virtual void    SendNoteOn(uint8_t Key, uint8_t Velocity, int32_t FragmentPos) = 0;
-            virtual void    SendNoteOff(uint8_t Key, uint8_t Velocity) = 0;
-            virtual void    SendNoteOff(uint8_t Key, uint8_t Velocity, int32_t FragmentPos) = 0;
-            virtual void    SendPitchbend(int Pitch) = 0;
-            virtual void    SendPitchbend(int Pitch, int32_t FragmentPos) = 0;
-            virtual void    SendControlChange(uint8_t Controller, uint8_t Value) = 0;
-            virtual void    SendControlChange(uint8_t Controller, uint8_t Value, int32_t FragmentPos) = 0;
+            virtual void    SendNoteOn(uint8_t Key, uint8_t Velocity, uint8_t MidiChannel) = 0;
+            virtual void    SendNoteOn(uint8_t Key, uint8_t Velocity, uint8_t MidiChannel, int32_t FragmentPos) = 0;
+            virtual void    SendNoteOff(uint8_t Key, uint8_t Velocity, uint8_t MidiChannel) = 0;
+            virtual void    SendNoteOff(uint8_t Key, uint8_t Velocity, uint8_t MidiChannel, int32_t FragmentPos) = 0;
+            virtual void    SendPitchbend(int Pitch, uint8_t MidiChannel) = 0;
+            virtual void    SendPitchbend(int Pitch, uint8_t MidiChannel, int32_t FragmentPos) = 0;
+            virtual void    SendControlChange(uint8_t Controller, uint8_t Value, uint8_t MidiChannel) = 0;
+            virtual void    SendControlChange(uint8_t Controller, uint8_t Value, uint8_t MidiChannel, int32_t FragmentPos) = 0;
             virtual void    SendProgramChange(uint8_t Program) = 0;
+            virtual void    SendChannelPressure(uint8_t Value, uint8_t MidiChannel) = 0;
+            virtual void    SendChannelPressure(uint8_t Value, uint8_t MidiChannel, int32_t FragmentPos) = 0;
+            virtual void    SendPolyphonicKeyPressure(uint8_t Key, uint8_t Value, uint8_t MidiChannel) = 0;
+            virtual void    SendPolyphonicKeyPressure(uint8_t Key, uint8_t Value, uint8_t MidiChannel, int32_t FragmentPos) = 0;
             virtual bool    StatusChanged(bool bNewStatus = false) = 0;
             virtual float   Volume() = 0;
             virtual void    Volume(float f) = 0;
             virtual float   Pan() = 0;
             virtual void    Pan(float f) = 0;
             virtual uint    Channels() = 0;
+
+            // audio driver management
             virtual void    Connect(AudioOutputDevice* pAudioOut) = 0;
             virtual void    DisconnectAudioOutputDevice() = 0;
             virtual AudioOutputDevice* GetAudioOutputDevice() = 0;
             virtual void    SetOutputChannel(uint EngineAudioChannel, uint AudioDeviceChannel) = 0;
             virtual int     OutputChannel(uint EngineAudioChannel) = 0;
-            virtual void    Connect(MidiInputPort* pMidiPort, midi_chan_t MidiChannel) = 0;
-            virtual void    DisconnectMidiInputPort() = 0;
-            virtual MidiInputPort* GetMidiInputPort() = 0;
+
+            // MIDI driver management
+            virtual void    Connect(MidiInputPort* pMidiPort) = 0;
+            virtual void    Disconnect(MidiInputPort* pMidiPort) = 0;
+            virtual void    DisconnectAllMidiInputPorts() = 0;
+            virtual uint    GetMidiInputPortCount() = 0;
+            virtual MidiInputPort* GetMidiInputPort(uint index) = 0;
             virtual midi_chan_t MidiChannel() = 0;
-            virtual String  InstrumentFileName() = 0;
+            virtual void    SetMidiChannel(midi_chan_t MidiChannel) = 0;
+            // (deprecated MIDI driver management methods)
+            virtual void    Connect(MidiInputPort* pMidiPort, midi_chan_t MidiChannel) DEPRECATED_API = 0;
+            virtual void    DisconnectMidiInputPort() DEPRECATED_API = 0;
+            virtual MidiInputPort* GetMidiInputPort() DEPRECATED_API = 0;
+
+            // virtual MIDI driver management (i.e. virtual on-screen MIDI keyboards)
+            virtual void    Connect(VirtualMidiDevice* pDevice) = 0;
+            virtual void    Disconnect(VirtualMidiDevice* pDevice) = 0;
+
+            // instrument (sound file) management
+            virtual void    PrepareLoadInstrument(const char* FileName, uint Instrument) = 0;
+            virtual void    LoadInstrument() = 0;
+            virtual String  InstrumentFileName() = 0; ///< Returns the file name of the currently loaded instrument. Equivalent as calling InstrumentFileName(0).
+            virtual String  InstrumentFileName(int index);
             virtual String  InstrumentName() = 0;
             virtual int     InstrumentIndex() = 0;
             virtual int     InstrumentStatus() = 0;
+
+            // sampler format / sampler engine implementation details
             virtual Engine* GetEngine() = 0;
             virtual String  EngineName() = 0;
+
+            // effect routing
             virtual FxSend* AddFxSend(uint8_t MidiCtrl, String Name = "") throw (Exception) = 0;
             virtual FxSend* GetFxSend(uint FxSendIndex) = 0;
             virtual uint    GetFxSendCount() = 0;
             virtual void    RemoveFxSend(FxSend* pFxSend) = 0;
-            virtual void    Connect(VirtualMidiDevice* pDevice) = 0;
-            virtual void    Disconnect(VirtualMidiDevice* pDevice) = 0;
 
 
             /////////////////////////////////////////////////////////////////
@@ -235,21 +260,122 @@ namespace LinuxSampler {
             void SetMidiInstrumentMap(int MidiMap) throw (Exception);
 
             /**
-             * Set MIDI Registered Parameter Number (RPN) Controller
-             * (upper 8 bits / coarse).
+             * Set MIDI Registered Parameter Number (RPN)
+             * (upper 7 bits / coarse).
              */
-            void SetMidiRpnControllerMsb(uint8_t CtrlMSB);
+            void SetMidiRpnParameterMsb(uint8_t ParamMSB);
 
             /**
-             * Set MIDI Registered Parameter Number (RPN) Controller
-             * (lower 8 bits / fine).
+             * Set MIDI Registered Parameter Number (RPN)
+             * (lower 7 bits / fine).
              */
-            void SetMidiRpnControllerLsb(uint8_t CtrlLSB);
+            void SetMidiRpnParameterLsb(uint8_t ParamLSB);
 
             /**
-             * Reset to no RPN controller currently selected.
+             * Set new MSB data value (upper 7 bits / coarse) for currently
+             * selected MIDI Registered Parameter Number (RPN).
              */
-            void ResetMidiRpnController();
+            void SetMidiRpnDataMsb(uint8_t DataMSB);
+
+            /**
+             * Set new LSB data value (lower 7 bits / fine) for currently
+             * selected MIDI Registered Parameter Number (RPN).
+             */
+            void SetMidiRpnDataLsb(uint8_t DataLSB);
+
+            /**
+             * Set new data value (14 bits, MSB and LSB data combined) for
+             * currently selected MIDI Registered Parameter Number (RPN).
+             */
+            void SetMidiRpnData(int Data);
+
+            /**
+             * Reset to no RPN parameter currently selected.
+             */
+            void ResetMidiRpnParameter();
+
+            /**
+             * Set MIDI Non-Registered Parameter Number (NRPN)
+             * (upper 7 bits / coarse).
+             */
+            void SetMidiNrpnParameterMsb(uint8_t ParamMSB);
+
+            /**
+             * Set MIDI Non-Registered Parameter Number (NRPN)
+             * (lower 7 bits / fine).
+             */
+            void SetMidiNrpnParameterLsb(uint8_t ParamLSB);
+
+            /**
+             * Set new MSB data value (upper 7 bits / coarse) for currently
+             * selected MIDI Non-Registered Parameter Number (NRPN).
+             */
+            void SetMidiNrpnDataMsb(uint8_t DataMSB);
+
+            /**
+             * Set new LSB data value (lower 7 bits / fine) for currently
+             * selected MIDI Non-Registered Parameter Number (NRPN).
+             */
+            void SetMidiNrpnDataLsb(uint8_t DataLSB);
+
+            /**
+             * Set new data value (14 bits, MSB and LSB data combined) for
+             * currently selected MIDI Non-Registered Parameter Number (NRPN).
+             */
+            void SetMidiNrpnData(int Data);
+
+            /**
+             * Reset to no NRPN parameter currently selected.
+             */
+            void ResetMidiNrpnParameter();
+
+            /**
+             * Get currently selected MIDI Registered Parameter Number
+             * (RPN), this method will return the already merged 14 bit
+             * parameter number (MSB and LSB value combined).
+             *
+             * @e WARNING: you have to call @c ResetMidiRpnParameter()
+             * after using this value, otherwise all subsequent MIDI CC #6
+             * (Data) messages are interpreted as RPN value messages.
+             *
+             * @returns currently selected (14 bit) RPN number, a negative
+             *          value if no RPN parameter currently selected
+             */
+            int GetMidiRpnParameter();
+
+            /**
+             * Get current data value of currently selected MIDI Registered
+             * Parameter Number (RPN), this method will return the already
+             * merged 14 bit data value (MSB and LSB data values combined).
+             *
+             * @returns latest data value (14 bits) of currently selected RPN
+             *          parameter, zero otherwise
+             */
+            int GetMidiRpnData();
+
+            /**
+             * Get currently selected MIDI Non-Registered Parameter Number
+             * (NRPN), this method will return the already merged 14 bit
+             * value (MSB and LSB value combined).
+             *
+             * @e WARNING: you have to call @c ResetMidiNrpnParameter()
+             * after using this value, otherwise all subsequent MIDI CC #6
+             * (Data) messages are interpreted as NRPN value messages.
+             *
+             * @returns currently selected (14 bit) NRPN number, a negative
+             *          value if no NRPN parameter currently selected
+             */
+            int GetMidiNrpnParameter();
+
+            /**
+             * Get current data value of currently selected MIDI Non-Registered
+             * Parameter Number (NRPN), this method will return the already
+             * merged 14 bit data value (MSB and LSB data values combined).
+             *
+             * @returns latest data value (14 bits) of currently selected NRPN
+             *          parameter, zero otherwise
+             */
+            int GetMidiNrpnData();
 
              /**
              * Registers the specified listener to be notified when the number
@@ -267,21 +393,6 @@ namespace LinuxSampler {
              */
             void RemoveAllFxSendCountListeners();
 
-            /**
-             * Get currently selected MIDI Registered Parameter Number
-             * (RPN) Controller, this method will return the already merged
-             * value (MSB and LSB value).
-             *
-             * @e WARNING: you have to call @c ResetMidiRpnController()
-             * after using this value, otherwise all subsequent MIDI CC #6
-             * (Data) messages are interpreted as RPN controller value
-             * messages.
-             *
-             * @returns currently selected RPN controller number, a negative
-             *          value if no RPN controller currently selected
-             */
-            int GetMidiRpnController();
-
             /**
              * Gets the current number of active voices.
              */
@@ -314,7 +425,91 @@ namespace LinuxSampler {
              *
              * This method is not real-time safe.
              */
-            void ExecuteProgramChange(uint8_t Program);
+            void ExecuteProgramChange(uint32_t Program);
+
+
+            // Deprecated APIs
+            // (these public methods might be removed in future) ...
+
+            /**
+             * Set MIDI Registered Parameter Number (RPN) Controller
+             * (upper 8 bits / coarse).
+             *
+             * @deprecated Use SetMidiRpnParameterMsb() instead.
+             */
+            void SetMidiRpnControllerMsb(uint8_t CtrlMSB) DEPRECATED_API;
+
+            /**
+             * Set MIDI Registered Parameter Number (RPN) Controller
+             * (lower 8 bits / fine).
+             *
+             * @deprecated Use SetMidiRpnParameterLsb() instead.
+             */
+            void SetMidiRpnControllerLsb(uint8_t CtrlLSB) DEPRECATED_API;
+
+            /**
+             * Reset to no RPN controller currently selected.
+             *
+             * @deprecated Use ResetMidiRpnParameter() instead.
+             */
+            void ResetMidiRpnController() DEPRECATED_API;
+
+            /**
+             * Set MIDI Non-Registered Parameter Number (NRPN) Controller
+             * (upper 8 bits / coarse).
+             *
+             * @deprecated Use SetMidiNrpnParameterMsb() instead.
+             */
+            void SetMidiNrpnControllerMsb(uint8_t CtrlMSB) DEPRECATED_API;
+
+            /**
+             * Set MIDI Non-Registered Parameter Number (NRPN) Controller
+             * (lower 8 bits / fine).
+             *
+             * @deprecated Use SetMidiNrpnParameterLsb() instead.
+             */
+            void SetMidiNrpnControllerLsb(uint8_t CtrlLSB) DEPRECATED_API;
+
+            /**
+             * Reset to no NRPN controller currently selected.
+             *
+             * @deprecated Use ResetMidiNrpnParameter() instead.
+             */
+            void ResetMidiNrpnController() DEPRECATED_API;
+
+            /**
+             * Get currently selected MIDI Registered Parameter Number
+             * (RPN) Controller, this method will return the already merged
+             * value (MSB and LSB value).
+             *
+             * @e WARNING: you have to call @c ResetMidiRpnController()
+             * after using this value, otherwise all subsequent MIDI CC #6
+             * (Data) messages are interpreted as RPN controller value
+             * messages.
+             *
+             * @returns currently selected RPN controller number, a negative
+             *          value if no RPN controller currently selected
+             *
+             * @deprecated Use GetMidiRpnParameter() instead.
+             */
+            int GetMidiRpnController() DEPRECATED_API;
+
+            /**
+             * Get currently selected MIDI Non-Registered Parameter Number
+             * (NRPN) Controller, this method will return the already merged
+             * value (MSB and LSB value).
+             *
+             * @e WARNING: you have to call @c ResetMidiNrpnController()
+             * after using this value, otherwise all subsequent MIDI CC #6
+             * (Data) messages are interpreted as NRPN controller value
+             * messages.
+             *
+             * @returns currently selected NRPN controller number, a negative
+             *          value if no NRPN controller currently selected
+             *
+             * @deprecated Use GetMidiNrpnParameter() instead.
+             */
+            int GetMidiNrpnController() DEPRECATED_API;
 
         protected:
             EngineChannel();
