@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2009 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2017 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -26,10 +26,15 @@
 
 #include "../../common/global.h"
 
-#include <gig.h>
+#if AC_APPLE_UNIVERSAL_BUILD
+# include <libgig/gig.h>
+#else
+# include <gig.h>
+#endif
 
-#include "../../common/global.h"
+#include "../../common/Pool.h"
 #include "../../common/ResourceManager.h"
+#include "../InstrumentManagerBase.h"
 #include "../../drivers/audio/AudioOutputDevice.h"
 #include "../InstrumentManager.h"
 #include "../../common/ArrayList.h"
@@ -42,15 +47,12 @@ namespace LinuxSampler { namespace gig {
 
 }} // namespace LinuxSampler::gig
 
-#include "EngineChannel.h"
-#include "Engine.h"
 #include "../../plugins/InstrumentEditor.h"
 
 namespace LinuxSampler { namespace gig {
 
-    // just symbol prototyping
-    class EngineChannel;
     class Engine;
+    class EngineChannel;
 
     /** @brief Gig instrument manager
      *
@@ -63,34 +65,27 @@ namespace LinuxSampler { namespace gig {
      * instrument / file is not needed anymore, then it will be freed from
      * memory.
      */
-    class InstrumentResourceManager : public InstrumentManager, public ResourceManager<InstrumentManager::instrument_id_t, ::gig::Instrument>, public InstrumentEditorListener {
+    class InstrumentResourceManager : public InstrumentManagerBase< ::gig::File, ::gig::Instrument, ::gig::DimensionRegion, ::gig::Sample>, public InstrumentEditorListener {
         public:
             InstrumentResourceManager() : Gigs(this) {}
             virtual ~InstrumentResourceManager() {}
             static void OnInstrumentLoadingProgress(::gig::progress_t* pProgress);
 
             // implementation of derived abstract methods from 'InstrumentManager'
-            virtual std::vector<instrument_id_t> Instruments();
-            virtual InstrumentManager::mode_t GetMode(const instrument_id_t& ID);
-            virtual void SetMode(const instrument_id_t& ID, InstrumentManager::mode_t Mode);
-            virtual String GetInstrumentName(instrument_id_t ID);
-            virtual String GetInstrumentDataStructureName(instrument_id_t ID);
-            virtual String GetInstrumentDataStructureVersion(instrument_id_t ID);
-            virtual InstrumentEditor* LaunchInstrumentEditor(instrument_id_t ID, void* pUserData = NULL) throw (InstrumentManagerException);
-            virtual std::vector<instrument_id_t> GetInstrumentFileContent(String File) throw (InstrumentManagerException);
-            virtual instrument_info_t GetInstrumentInfo(instrument_id_t ID) throw (InstrumentManagerException);
+            virtual String GetInstrumentName(instrument_id_t ID) OVERRIDE;
+            virtual String GetInstrumentDataStructureName(instrument_id_t ID) OVERRIDE;
+            virtual String GetInstrumentDataStructureVersion(instrument_id_t ID) OVERRIDE;
+            virtual InstrumentEditor* LaunchInstrumentEditor(LinuxSampler::EngineChannel* pEngineChannel, instrument_id_t ID, void* pUserData = NULL) throw (InstrumentManagerException) OVERRIDE;
+            virtual std::vector<instrument_id_t> GetInstrumentFileContent(String File) throw (InstrumentManagerException) OVERRIDE;
+            virtual instrument_info_t GetInstrumentInfo(instrument_id_t ID) throw (InstrumentManagerException) OVERRIDE;
 
             // implementation of derived abstract methods from 'InstrumentEditorListener'
-            virtual void OnInstrumentEditorQuit(InstrumentEditor* pSender);
-            virtual void OnSamplesToBeRemoved(std::set<void*> Samples, InstrumentEditor* pSender);
-            virtual void OnSamplesRemoved(InstrumentEditor* pSender);
-            virtual void OnDataStructureToBeChanged(void* pStruct, String sStructType, InstrumentEditor* pSender);
-            virtual void OnDataStructureChanged(void* pStruct, String sStructType, InstrumentEditor* pSender);
-            virtual void OnSampleReferenceChanged(void* pOldSample, void* pNewSample, InstrumentEditor* pSender);
-
-            void HandBackInstrument(::gig::Instrument* pResource, InstrumentConsumer* pConsumer,
-                                    RTList< ::gig::DimensionRegion*>* pDimRegionsInUse);
-            void HandBackDimReg(::gig::DimensionRegion* pDimReg);
+            virtual void OnInstrumentEditorQuit(InstrumentEditor* pSender) OVERRIDE;
+            virtual void OnSamplesToBeRemoved(std::set<void*> Samples, InstrumentEditor* pSender) OVERRIDE;
+            virtual void OnSamplesRemoved(InstrumentEditor* pSender) OVERRIDE;
+            virtual void OnDataStructureToBeChanged(void* pStruct, String sStructType, InstrumentEditor* pSender) OVERRIDE;
+            virtual void OnDataStructureChanged(void* pStruct, String sStructType, InstrumentEditor* pSender) OVERRIDE;
+            virtual void OnSampleReferenceChanged(void* pOldSample, void* pNewSample, InstrumentEditor* pSender) OVERRIDE;
 
 #if 0 // currently unused :
             void TrySendNoteOnToEditors(uint8_t Key, uint8_t Velocity, ::gig::Instrument* pInstrument);
@@ -99,18 +94,23 @@ namespace LinuxSampler { namespace gig {
 
         protected:
             // implementation of derived abstract methods from 'ResourceManager'
-            virtual ::gig::Instrument* Create(instrument_id_t Key, InstrumentConsumer* pConsumer, void*& pArg);
-            virtual void               Destroy(::gig::Instrument* pResource, void* pArg);
-            virtual void               OnBorrow(::gig::Instrument* pResource, InstrumentConsumer* pConsumer, void*& pArg);
+            virtual ::gig::Instrument* Create(instrument_id_t Key, InstrumentConsumer* pConsumer, void*& pArg) OVERRIDE;
+            virtual void               Destroy(::gig::Instrument* pResource, void* pArg) OVERRIDE;
+            virtual void               DeleteRegionIfNotUsed(::gig::DimensionRegion* pRegion, region_info_t* pRegInfo) OVERRIDE;
+            virtual void               DeleteSampleIfNotUsed(::gig::Sample* pSample, region_info_t* pRegInfo) OVERRIDE;
         private:
+            void                       CacheInitialSamples(::gig::Sample* pSample, AbstractEngine* pEngine);
+            void                       CacheInitialSamples(::gig::Sample* pSample, EngineChannel* pEngineChannel);
+            void                       CacheInitialSamples(::gig::Sample* pSample, uint maxSamplesPerCycle);
+
             typedef ResourceConsumer< ::gig::File> GigConsumer;
 
             class GigResourceManager : public ResourceManager<String, ::gig::File> {
                 protected:
                     // implementation of derived abstract methods from 'ResourceManager'
-                    virtual ::gig::File* Create(String Key, GigConsumer* pConsumer, void*& pArg);
-                    virtual void         Destroy(::gig::File* pResource, void* pArg);
-                    virtual void         OnBorrow(::gig::File* pResource, GigConsumer* pConsumer, void*& pArg) {} // ignore
+                    virtual ::gig::File* Create(String Key, GigConsumer* pConsumer, void*& pArg) OVERRIDE;
+                    virtual void         Destroy(::gig::File* pResource, void* pArg) OVERRIDE;
+                    virtual void         OnBorrow(::gig::File* pResource, GigConsumer* pConsumer, void*& pArg) OVERRIDE {} // ignore
                 public:
                     GigResourceManager(InstrumentResourceManager* parent) : parent(parent) {}
                     virtual ~GigResourceManager() {}
@@ -118,31 +118,23 @@ namespace LinuxSampler { namespace gig {
                     InstrumentResourceManager* parent;
             } Gigs;
 
-            void CacheInitialSamples(::gig::Sample* pSample, gig::EngineChannel* pEngineChannel);
-            void CacheInitialSamples(::gig::Sample* pSample, gig::Engine* pEngine);
             void UncacheInitialSamples(::gig::Sample* pSample);
             std::vector< ::gig::Instrument*> GetInstrumentsCurrentlyUsedOf(::gig::File* pFile, bool bLock);
-            std::set<gig::EngineChannel*> GetEngineChannelsUsing(::gig::Instrument* pInstrument, bool bLock);
-            std::set<gig::Engine*> GetEnginesUsing(::gig::Instrument* pInstrument, bool bLock);
-            std::set<gig::Engine*> GetEnginesUsing(::gig::File* pFile, bool bLock);
+            std::set<EngineChannel*> GetEngineChannelsUsingScriptSourceCode(const String& code, bool bLock);
+            std::set<EngineChannel*> GetEngineChannelsUsing(::gig::Instrument* pInstrument, bool bLock);
+            std::set<Engine*> GetEnginesUsing(::gig::Instrument* pInstrument, bool bLock);
+            std::set<Engine*> GetEnginesUsing(::gig::File* pFile, bool bLock);
             bool SampleReferencedByInstrument(::gig::Sample* pSample, ::gig::Instrument* pInstrument);
             void SuspendEnginesUsing(::gig::Instrument* pInstrument);
             void SuspendEnginesUsing(::gig::File* pFile);
             void ResumeAllEngines();
 
-            struct dimreg_info_t {
-                int           refCount;
-                ::gig::File*  file;
-                ::RIFF::File* riff;
-            };
-            Mutex DimRegInfoMutex; ///< protects the DimRegInfo and SampleRefCount maps from concurrent access by the instrument loader and disk threads
-            std::map< ::gig::DimensionRegion*, dimreg_info_t> DimRegInfo; ///< contains dimension regions that are still in use but belong to released instrument
-            std::map< ::gig::Sample*, int> SampleRefCount; ///< contains samples that are still in use but belong to a released instrument
-
             Mutex InstrumentEditorProxiesMutex; ///< protects the 'InstrumentEditorProxies' map
             ArrayList<InstrumentConsumer*> InstrumentEditorProxies; ///< here we store the objects that react on instrument specific notifications on behalf of the respective instrument editor
             std::set<Engine*> suspendedEngines; ///< all engines currently completely suspended
             Mutex             suspendedEnginesMutex; ///< protects 'suspendedEngines' set
+            std::map< ::gig::Script*,String> pendingScriptUpdates; ///< Used to prepare updates of instrument scripts (value of the map is the original source code of the script before it is modified).
+            Mutex                            pendingScriptUpdatesMutex; ///< Protectes 'pendingScriptUpdates'.
     };
 
 }} // namespace LinuxSampler::gig
